@@ -4,9 +4,9 @@
  * authors: Tomasz Mrugalski <thomson@klub.com.pl>
  *          Marek Senderski  <msend@o2.pl>
  *
- * released under GNU GPL v2 or later licence
+ * released under GNU GPL v2 only licence
  *
- * $Id: AddrIA.cpp,v 1.21 2007-12-09 04:08:35 thomson Exp $
+ * $Id: AddrIA.cpp,v 1.24 2008-11-11 21:55:27 thomson Exp $
  *
  */
 
@@ -30,7 +30,7 @@ TAddrIA::TAddrIA(int iface, SmartPtr<TIPv6Addr> addr, SmartPtr<TDUID> duid,
     this->T2 = T2;
     this->IAID = ID;
     this->Tentative = TENTATIVE_UNKNOWN;
-    this->State     = STATE_NOTCONFIGURED;
+    this->State     = STATE_NOTCONFIGURED;//	STATE_CONFIRMME;
     this->Timestamp = now();
     this->Unicast = false;
     this->setDUID(duid);
@@ -46,6 +46,11 @@ unsigned long TAddrIA::getIAID()
     return this->IAID;
 }
 
+void TAddrIA::reset()
+{
+    setState(STATE_NOTCONFIGURED);
+}
+
 int TAddrIA::getIface()
 {
     return this->Iface;
