@@ -6,9 +6,9 @@
  * changes: Krzysztof Wnuk <keczi@poczta.onet.pl>
  *          Micha Kowalczuk <michal@kowalczuk.eu>
  *
- * released under GNU GPL v2 or later licence
+ * released under GNU GPL v2 only licence
  *
- * $Id: AddrMgr.cpp,v 1.31 2008-02-25 17:49:06 thomson Exp $
+ * $Id: AddrMgr.cpp,v 1.37 2008-08-29 00:07:26 thomson Exp $
  *
  */
 
@@ -18,6 +18,7 @@
 #include <limits.h>
 #include "AddrMgr.h"
 #include "AddrClient.h"
+#include "DHCPConst.h"
 #include "Logger.h"
 
 TAddrMgr::TAddrMgr(string xmlFile, bool loadfile)
@@ -25,23 +26,31 @@ TAddrMgr::TAddrMgr(string xmlFile, bool loadfile)
     this->IsDone = false;
     this->XmlFile = xmlFile;
     
-    if (loadfile)
-	dbLoad();
+    if (loadfile) {
+	dbLoad(CLNTADDRMGR_FILE);
+    } else {
+	Log(Debug) << "Skipping database loading." << LogEnd;
+    }
 
     DeleteEmptyClient = true;
 }
 
-void TAddrMgr::dbLoad()
+void TAddrMgr::dbLoad(const char * xmlFile)
 {
 
-#ifdef LIBXML2
+#ifdef MOD_LIBXML2
+    Log(Debug) << "Loading " << xmlFile << " (using libxml2)." << LogEnd;
      xmlDocPtr root;
-     root = xmlLoad(this->dbfile.c_str());
+     root = xmlLoad(xmlFile);
      if (!root) {
  	Log(Error) << "File loading has failed." << LogEnd;
+	return;
      }
      this->parseAddrMgr(root,0);
      xmlFreeDoc(root);
+#else
+     Log(Info) << "Loading old address database (" << xmlFile << "), using built-in routines." << LogEnd;
+     xmlLoadBuiltIn(xmlFile);
 #endif
 }
 
@@ -457,9 +466,9 @@ bool TAddrMgr::prefixIsFree(SPtr<TIPv6Addr> x)
 /* PD-ends */
 
 // --------------------------------------------------------------------
-// --- XML-related methods --------------------------------------------
+// --- XML-related methods (libxml2) ----------------------------------
 // --------------------------------------------------------------------
-#ifdef LIBXML2
+#ifdef MOD_LIBXML2
 // loads entire file
 xmlDocPtr TAddrMgr::xmlLoad(const char * filename) {
      xmlDocPtr doc;
@@ -491,7 +500,7 @@ xmlDocPtr TAddrMgr::xmlLoad(const char * filename) {
      if (!dtd)
      {
  	Log(Error) << "DTD load failed." << LogEnd;
- 	return NULL;
+ 	//return NULL;
      }
 
      return doc;
@@ -540,8 +549,11 @@ SmartPtr<TAddrIA> TAddrMgr::parseAddrIA(xmlDocPtr doc, xmlNodePtr xmlIA, int dep
      // unicast
      xmlChar * unicast = xmlGetProp(xmlIA,(const xmlChar*)"unicast");
      char packedUnicast[16];
-     inet_pton6((const char *)unicast,packedUnicast);
-     SmartPtr<TIPv6Addr> ptrUnicast = new TIPv6Addr(packedUnicast);
+     SmartPtr<TIPv6Addr> ptrUnicast;
+     if (unicast) {
+       inet_pton6((const char *)unicast,packedUnicast);
+       ptrUnicast = new TIPv6Addr(packedUnicast);
+     }
 
      // T1
      xmlChar * T1Str = xmlGetProp(xmlIA,(const xmlChar*)"T1");
@@ -581,6 +593,7 @@ SmartPtr<TAddrIA> TAddrMgr::parseAddrIA(xmlDocPtr doc, xmlNodePtr xmlIA, int dep
  	if (xmlAddr->type == XML_ELEMENT_NODE) {
  	    ptrAddr = parseAddrAddr(doc, xmlAddr, depth+2);
  	    ptrIA->addAddr(ptrAddr);
+	    ptrAddr->setTentative(TENTATIVE_NO);
  	}
  	xmlAddr = xmlAddr->next;
      }
@@ -634,6 +647,185 @@ void TAddrMgr::parseAddrMgr(xmlDocPtr doc,int depth)
  	xmlClient = xmlClient->next;
      }
 }
+#else
+// --------------------------------------------------------------------
+// --- XML-related methods (built-in) ---------------------------------
+// --------------------------------------------------------------------
+bool TAddrMgr::xmlLoadBuiltIn(const char * xmlFile)
+{
+    SPtr<TAddrClient> clnt;
+    bool AddrClient = false;
+
+    char buf[256];
+
+    FILE * f;
+    if (!(f = fopen(xmlFile,"r"))) {
+        Log(Warning) << "Unable to open " << xmlFile << "." << LogEnd;
+        return false;
+    }
+
+    while (!feof(f)) {
+      fgets(buf, 255, f);
+	if (strstr(buf,"<AddrMgr>")) {
+	    AddrClient = true;
+	    continue;
+	}
+	if (AddrClient && strstr(buf,"<AddrClient")) {
+	  clnt = parseAddrClient(f);
+	  continue;
+	}
+
+	if (strstr(buf,"</AddrMgr>")) {
+	    break;
+	}
+    }
+    fclose(f);
+
+    if (clnt) {
+	Log(Notice) << "Address database loaded successfuly." << LogEnd;
+	ClntsLst.append(clnt);
+	return true;
+    } 
+    
+    return false;
+}
+
+SPtr<TAddrClient> TAddrMgr::parseAddrClient(FILE *f)
+{
+    char buf[256];
+    char * x = 0;
+    SPtr<TAddrClient> clnt = 0;
+    SPtr<TDUID> duid = 0;
+    SPtr<TAddrIA> ia = 0;
+
+    while (!feof(f)) {
+	fgets(buf,255,f);
+	if (strstr(buf,"<duid")) {
+	    x = strstr(buf,">")+1;
+	    x = strstr(x,"</duid>");
+	    if (x)
+		*x = 0; // remove trailing xml tag
+	    duid = new TDUID(strstr(buf,">")+1);
+	    clnt = new TAddrClient(duid);
+	    // Log(Debug) << "New address loaded from file created: duid=" << duid->getPlain() << LogEnd;
+	    if (ia = parseAddrIA(f)) {
+		clnt->addIA(ia);
+	    }
+	    // FIXME: support for more than one IA
+
+	    // FIXME: support for PD
+	    continue;
+	}
+	if (strstr(buf,"</AddrClient>"))
+	    break;
+    }
+
+    // FIXME: add some extra checks here
+    return clnt;
+}
+
+SPtr<TAddrIA> TAddrMgr::parseAddrIA(FILE * f)
+{
+    // IA paramteres
+    char buf[256];
+    char * x = 0;
+    int t1 = 0, t2 = 0, iaid = 0, iface = 0;
+    SPtr<TAddrIA> ia = 0;
+    SPtr<TAddrAddr> addr;
+    SPtr<TDUID> duid;
+
+    while (!feof(f)) {
+	fgets(buf,255,f);
+	if (strstr(buf,"<AddrIA")) {
+	    if ((x=strstr(buf,"T1"))) {
+		t1=atoi(x+4);
+	    }
+	    if ((x=strstr(buf,"T2"))) {
+		t2=atoi(x+4);
+	    }
+	    if ((x=strstr(buf,"IAID"))) {
+		iaid=atoi(x+6);
+	    }
+	    if ((x=strstr(buf,"iface"))) {
+		iface=atoi(x+7);
+	    }
+
+	    if (t1!=0 && t2!=0 && iaid!=0 && iface!=0) {
+		Log(Debug) << "Loaded IA from a file: t1=" << t1 << ", t2="<< t2 << ",iaid=" << iaid << ", iface=" << iface << LogEnd;
+		duid = 0; // don't use old DUID
+	    }
+	    continue;
+	}
+	if (strstr(buf,"duid")) {
+	    char * x;
+	    x = strstr(buf,">")+1;
+	    x = strstr(x,"</duid>");
+	    if (x)
+		*x = 0; // remove trailing xml tag
+	    duid = new TDUID(strstr(buf,">")+1);
+	    // Log(Debug) << "Parsed IA: duid=" << duid->getPlain() << LogEnd;
+
+	    ia = new TAddrIA(iface, 0, duid, t1,t2, iaid);
+	    ia->setState(STATE_CONFIRMME);
+	    continue;
+	}
+	if (strstr(buf,"<AddrAddr")) {
+	    addr = parseAddrAddr(buf);
+	    if (ia && addr) {
+		ia->addAddr(addr);
+		addr->setTentative(TENTATIVE_NO);
+	    }
+	}
+	if (strstr(buf,"</AddrIA>"))
+	    break;
+
+    }
+    if (ia)
+	ia->setTentative();
+    return ia;
+}
+
+SPtr<TAddrAddr> TAddrMgr::parseAddrAddr(char * buf)
+{
+    // address parameters
+    unsigned long timestamp, pref=DHCPV6_INFINITY, valid=DHCPV6_INFINITY, prefix = CLIENT_DEFAULT_PREFIX_LENGTH;
+    SPtr<TIPv6Addr> addr = 0;
+    SPtr<TAddrAddr> addraddr;
+    char * x;
+
+    
+
+    if (strstr(buf, "<AddrAddr")) {
+	timestamp=pref=valid=0;
+	addr = 0;
+	if ((x=strstr(buf,"timestamp"))) {
+	    timestamp = atoi(x+11);
+	}
+	if ((x=strstr(buf,"pref="))) {
+	    pref = atoi(x+6);
+	}
+	if ((x=strstr(buf,"valid"))) {
+	    valid = atoi(x+7);
+	}
+	if ((x=strstr(buf,"prefix="))) {
+	    prefix = atoi(x+8);
+	}
+	if ((x=strstr(buf,">"))) {
+	    x = strstr(x, "</AddrAddr>");
+	    if (x)
+		*x = 0;
+	    addr = new TIPv6Addr(strstr(buf,">")+1, true);
+	    Log(Debug) << "Parsed addr=" << addr->getPlain() << ", pref=" << pref << ", valid=" << valid << ",ts=" << timestamp << LogEnd;
+
+	}
+	if (addr && timestamp && pref && valid) {
+	    addraddr = new TAddrAddr(addr, pref, valid, prefix);
+	    addraddr->setTimestamp(timestamp);
+	}
+    }
+    return addraddr;
+}
+
 #endif
 
 bool TAddrMgr::isDone() {
