@@ -4,9 +4,9 @@
  * authors: Tomasz Mrugalski <thomson@klub.com.pl>
  *          Marek Senderski <msend@o2.pl>
  *
- * released under GNU GPL v2 or later licence
+ * released under GNU GPL v2 only licence
  *
- * $Id: ClntAddrMgr.cpp,v 1.21 2007-05-04 17:20:39 thomson Exp $
+ * $Id: ClntAddrMgr.cpp,v 1.26 2008-11-11 22:02:43 thomson Exp $
  */
 
 #include "SmartPtr.h"
@@ -18,15 +18,18 @@
 
 
 TClntAddrMgr::TClntAddrMgr(SmartPtr<TClntCfgMgr> ClntCfgMgr, string xmlFile, bool loadDB)
-    :TAddrMgr(xmlFile, loadDB)
+    :TAddrMgr(xmlFile, ClntCfgMgr->useConfirm())
 {
-    
-    // add this client (with proper duid)
-    SmartPtr<TAddrClient> client = new TAddrClient(ClntCfgMgr->getDUID());
-    addClient(client);
-    DeleteEmptyClient = false; // don't delete this client, even when IAs or PD has been removed
+    // client may have been already loaded from client-AddrMgr.xml file
+    firstClient();
+    if (!getClient()) {
+	// add this client (with proper duid)
+	SmartPtr<TAddrClient> client = new TAddrClient(ClntCfgMgr->getDUID());
+	addClient(client);
+    }
 
     // set Client field
+    DeleteEmptyClient = false; // don't delete this client, even when IAs or PD has been removed
     firstClient();
     Client = getClient();
 }
@@ -149,6 +152,37 @@ SmartPtr<TAddrIA> TClntAddrMgr::getIA(unsigned long IAID)
     return 0;
 }
 
+/** 
+ * sets specified interface to CONFIRMME state
+ * 
+ * @param changedLinks structure containing interface indexes to be confirmed
+ */
+void TClntAddrMgr::setIA2Confirm(volatile link_state_notify_t * changedLinks)
+{
+    SmartPtr<TAddrIA> ptrIA;
+    this->firstIA();
+    while(ptrIA = this->getIA()){
+
+	bool found = false;
+	int ifindex = ptrIA->getIface(); // interface index of this IA
+
+	// is this index on the list of interfaces to be confirmed?
+	for (int i=0; i < MAX_LINK_STATE_CHANGES_AT_ONCE; i++)
+	    if (changedLinks->ifindex[i]==ifindex)
+		found = true;
+
+	if (!found)
+	    continue;
+
+        if( (ptrIA->getState() == STATE_CONFIGURED || ptrIA->getState() == STATE_INPROCESS) )
+	{
+	    ptrIA->setState(STATE_CONFIRMME);
+	    Log(Notice) << "Network switch off event detected. do Confirmming." << LogEnd;
+	}
+    } 
+
+}
+
 // pd functions 
 
 void TClntAddrMgr::firstPD() {
