@@ -4,9 +4,9 @@
  * authors: Tomasz Mrugalski <thomson@klub.com.pl>
  *          Marek Senderski <msend@o2.pl>
  *
- * released under GNU GPL v2 or later licence
+ * released under GNU GPL v2 only licence
  *
- * $Id: ClntAddrMgr.h,v 1.11 2007-02-02 00:52:03 thomson Exp $
+ * $Id: ClntAddrMgr.h,v 1.14 2008-10-12 14:05:25 thomson Exp $
  *
  */
 
@@ -18,6 +18,7 @@
 #include "AddrIA.h"
 #include "AddrMgr.h"
 #include "ClntCfgMgr.h"
+#include "Portable.h"
 
 class TClntAddrMgr : public TAddrMgr
 {
@@ -39,6 +40,8 @@ class TClntAddrMgr : public TAddrMgr
     void addIA(SmartPtr<TAddrIA> ptr);
     bool delIA(long IAID);
     int countIA();
+    // CHANGED here: when network switch off signal received, the funtion will be invoked to set valid IA to CONFIRMME state.
+    void setIA2Confirm(volatile link_state_notify_t * changedLinks);
 
     // --- PD --- 
     void firstPD();
