@@ -6,9 +6,9 @@
  * changes: Krzysztof Wnuk <keczi@poczta.onet.pl>                                                                         
  *          Michal Kowalczuk <michal@kowalczuk.eu>
  * 
- * released under GNU GPL v2 or later licence                                
+ * released under GNU GPL v2 only licence                                
  *                                                                           
- * $Id: ClntCfgMgr.cpp,v 1.58 2008-06-02 00:15:00 thomson Exp $
+ * $Id: ClntCfgMgr.cpp,v 1.66 2008-09-22 17:08:52 thomson Exp $
  *
  */
 
@@ -43,6 +43,7 @@ TClntCfgMgr::TClntCfgMgr(SmartPtr<TClntIfaceMgr> ClntIfaceMgr,
 {
     this->IfaceMgr = ClntIfaceMgr;
     this->IsDone=false;
+    NotifyScripts = false;
 
     if (!parseConfigFile(cfgFile)) {
 	this->IsDone = true;
@@ -188,7 +189,7 @@ bool TClntCfgMgr::matchParsedSystemInterfaces(ClntParser *parser) {
 	    // Check if the interface is during bring-up phase (i.e. DAD procedure for link-local addr is not complete yet)
 	    char tmp[64];
 	    ifaceIface->firstLLAddress();
-	    inet_ntop6(ifaceIface->getLLAddress(), tmp); 
+	    inet_ntop6(ifaceIface->getLLAddress(), tmp);
 	    if (is_addr_tentative(ifaceIface->getName(), ifaceIface->getID(), tmp) == LOWLEVEL_TENTATIVE_YES) {
 		Log(Notice) << "Interface " << ifaceIface->getFullName() << " has link-local address " << tmp 
 			    << ", but it is currently tentative." << LogEnd;
@@ -216,7 +217,12 @@ bool TClntCfgMgr::matchParsedSystemInterfaces(ClntParser *parser) {
 	// we'll try to configure each interface we could find
 	Log(Warning) << "Config file does not contain any interface definitions. Trying to autodetect."
 		     << LogEnd;
+
+	List(TIPv6Addr) dnsList;
+	dnsList.clear();
+	parser->ParserOptStack.getLast()->setDNSServerLst(&dnsList);
 	
+	int cnt = 0;
 	IfaceMgr->firstIface();
 	while ( ifaceIface = IfaceMgr->getIface() ) {
 	    // for each interface present in the system...
@@ -268,6 +274,12 @@ bool TClntCfgMgr::matchParsedSystemInterfaces(ClntParser *parser) {
 
 	    Log(Info) << "Interface " << cfgIface->getName() << "/" << cfgIface->getID() 
                       << " has been added." << LogEnd;
+	    cnt ++;
+	}
+	if (!cnt) {
+	    Log(Crit) << "Unable to detect any suitable interfaces. If there are any interfaces that you"
+		      << " want to have configured, please specify them in client.conf file." << LogEnd;
+	    return false;
 	}
     }
     return true;
@@ -519,6 +531,7 @@ bool TClntCfgMgr::setGlobalOptions(ClntParser * parser)
     this->LogLevel       = logger::getLogLevel();
     this->LogName        = logger::getLogName();
     this->ScriptsDir     = opt->getScriptsDir();
+    this->NotifyScripts  = opt->getNotifyScripts();
     this->AnonInfRequest = opt->getAnonInfRequest();
     this->InsistMode     = opt->getInsistMode();   // should the client insist on receiving all options
                                                    // i.e. sending INF-REQUEST if REQUEST did not grant required opts
@@ -527,6 +540,10 @@ bool TClntCfgMgr::setGlobalOptions(ClntParser * parser)
     this->FQDNFlagS      = opt->getFQDNFlagS();
 
     this->MappingPrefix  = opt->getMappingPrefix(); // experimental feature
+
+    this->UseConfirm     = opt->getConfirm(); // should client try to send CONFIRM?
+
+    this->TunnelMode     = opt->getTunnelMode();
     
     // user has specified DUID type, just in case if new DUID will be generated
     if (parser->DUIDType != DUID_TYPE_NOT_DEFINED) {
@@ -668,6 +685,15 @@ bool TClntCfgMgr::getMappingPrefix()
     return MappingPrefix;
 }
 
+bool TClntCfgMgr::useConfirm()
+{
+    return UseConfirm;
+}
+
+int TClntCfgMgr::tunnelMode()
+{
+    return TunnelMode;
+}
 
 TClntCfgMgr::~TClntCfgMgr() {
     Log(Debug) << "ClntCfgMgr cleanup." << LogEnd;
@@ -713,6 +739,10 @@ ostream & operator<<(ostream &strum, TClntCfgMgr &x)
     return strum;
 }
 
+bool TClntCfgMgr::getNotifyScripts()
+{
+    return NotifyScripts;
+}
 
 #ifdef MOD_CLNT_EMBEDDED_CFG
 /** 
