@@ -4,9 +4,9 @@
  * authors: Tomasz Mrugalski <thomson@klub.com.pl>                           
  *          Marek Senderski <msend@o2.pl>                                    
  *                                                                           
- * released under GNU GPL v2 or later licence                                
+ * released under GNU GPL v2 only licence                                
  *                                                                           
- * $Id: ClntParsGlobalOpt.h,v 1.15 2008-06-01 23:15:34 thomson Exp $
+ * $Id: ClntParsGlobalOpt.h,v 1.19 2008-08-30 21:41:11 thomson Exp $
  *
  */
 
@@ -31,6 +31,8 @@ public:
     DigestTypes getDigest();
     void setScriptsDir(string dir);
     string getScriptsDir();
+    void setNotifyScripts(bool useScripts);
+    bool getNotifyScripts();
     void setAnonInfRequest(bool anonymous);
     bool getAnonInfRequest();
     void setInsistMode(bool insist);
@@ -52,21 +54,29 @@ public:
     void setFQDNFlagS(bool s);
     bool getFQDNFlagS();
 
+    void setConfirm(bool conf);
+    bool getConfirm();
+
+    void setTunnelMode(int vendorid);
+    int  getTunnelMode();
 private:
     string WorkDir;
     int PrefixLength;
     DigestTypes Digest;
     string ScriptsDir;
+    bool   NotifyScripts;
     bool   AnonInfRequest;
     bool   InactiveMode;
     bool   InsistMode;
     bool   AuthEnabled;
     List(DigestTypes) AuthAcceptMethods;
+    bool UseConfirm;
 
     bool FQDNFlagS;
 
     bool Experimental;
     bool ExperimentalMappingPrefix;
+    int  TunnelMode;
 };
 
 #endif
