@@ -51,6 +51,7 @@ TContainer<SmartPtr<TStationID> > PresentStationLst;                        \
 TContainer<SmartPtr<TIPv6Addr> > PresentAddrLst;                            \
 TContainer<SmartPtr<string> > PresentStringLst;                             \
 List(TClntOptVendorSpec) VendorSpec;					    \
+SPtr<TClntOptVendorSpec> TunnelMode;                                        \
 /*method check whether interface with id=ifaceNr has been */                \
 /*already declared */                                                       \
 bool CheckIsIface(int ifaceNr);                                             \
@@ -78,9 +79,10 @@ SPtr<TDUID> DUIDEnterpriseID;
     ParserOptStack.getFirst()->setIAIDCnt(1);                               \
     ParserOptStack.getLast();                                               \
     DUIDType = DUID_TYPE_NOT_DEFINED;                                       \
-    DUIDEnterpriseID = 0;
+    DUIDEnterpriseID = 0;                                                   \
+    TunnelMode = 0;
 
-#line 79 "ClntParser.y"
+#line 81 "ClntParser.y"
 typedef union    
 {
     int ival;    
@@ -292,36 +294,39 @@ typedef
 #define	RAPID_COMMIT_	287
 #define	OPTION_	288
 #define	SCRIPTS_DIR_	289
-#define	LOGNAME_	290
-#define	LOGLEVEL_	291
-#define	LOGMODE_	292
-#define	STRING_	293
-#define	HEXNUMBER_	294
-#define	INTNUMBER_	295
-#define	DUID_	296
-#define	STRICT_RFC_NO_ROUTING_	297
-#define	PD_	298
-#define	DUID_TYPE_	299
-#define	DUID_TYPE_LLT_	300
-#define	DUID_TYPE_LL_	301
-#define	DUID_TYPE_EN_	302
-#define	AUTH_ENABLED_	303
-#define	AUTH_ACCEPT_METHODS_	304
-#define	DIGEST_NONE_	305
-#define	DIGEST_PLAIN_	306
-#define	DIGEST_HMAC_MD5_	307
-#define	DIGEST_HMAC_SHA1_	308
-#define	DIGEST_HMAC_SHA224_	309
-#define	DIGEST_HMAC_SHA256_	310
-#define	DIGEST_HMAC_SHA384_	311
-#define	DIGEST_HMAC_SHA512_	312
-#define	STATELESS_	313
-#define	ANON_INF_REQUEST_	314
-#define	INSIST_MODE_	315
-#define	INACTIVE_MODE_	316
-#define	EXPERIMENTAL_	317
-#define	ADDR_PARAMS_	318
-#define	MAPPING_PREFIX_	319
+#define	NOTIFY_SCRIPTS_	290
+#define	LOGNAME_	291
+#define	LOGLEVEL_	292
+#define	LOGMODE_	293
+#define	STRING_	294
+#define	HEXNUMBER_	295
+#define	INTNUMBER_	296
+#define	DUID_	297
+#define	STRICT_RFC_NO_ROUTING_	298
+#define	SKIP_CONFIRM_	299
+#define	PD_	300
+#define	DUID_TYPE_	301
+#define	DUID_TYPE_LLT_	302
+#define	DUID_TYPE_LL_	303
+#define	DUID_TYPE_EN_	304
+#define	AUTH_ENABLED_	305
+#define	AUTH_ACCEPT_METHODS_	306
+#define	DIGEST_NONE_	307
+#define	DIGEST_PLAIN_	308
+#define	DIGEST_HMAC_MD5_	309
+#define	DIGEST_HMAC_SHA1_	310
+#define	DIGEST_HMAC_SHA224_	311
+#define	DIGEST_HMAC_SHA256_	312
+#define	DIGEST_HMAC_SHA384_	313
+#define	DIGEST_HMAC_SHA512_	314
+#define	STATELESS_	315
+#define	ANON_INF_REQUEST_	316
+#define	INSIST_MODE_	317
+#define	INACTIVE_MODE_	318
+#define	EXPERIMENTAL_	319
+#define	ADDR_PARAMS_	320
+#define	MAPPING_PREFIX_	321
+#define	TUNNEL_MODE_	322
 
 
 #line 169 "../bison++/bison.h"
@@ -402,6 +407,7 @@ static const int WORKDIR_;
 static const int RAPID_COMMIT_;
 static const int OPTION_;
 static const int SCRIPTS_DIR_;
+static const int NOTIFY_SCRIPTS_;
 static const int LOGNAME_;
 static const int LOGLEVEL_;
 static const int LOGMODE_;
@@ -410,6 +416,7 @@ static const int HEXNUMBER_;
 static const int INTNUMBER_;
 static const int DUID_;
 static const int STRICT_RFC_NO_ROUTING_;
+static const int SKIP_CONFIRM_;
 static const int PD_;
 static const int DUID_TYPE_;
 static const int DUID_TYPE_LLT_;
@@ -432,6 +439,7 @@ static const int INACTIVE_MODE_;
 static const int EXPERIMENTAL_;
 static const int ADDR_PARAMS_;
 static const int MAPPING_PREFIX_;
+static const int TUNNEL_MODE_;
 
 
 #line 212 "../bison++/bison.h"
@@ -472,36 +480,39 @@ static const int MAPPING_PREFIX_;
 	,RAPID_COMMIT_=287
 	,OPTION_=288
 	,SCRIPTS_DIR_=289
-	,LOGNAME_=290
-	,LOGLEVEL_=291
-	,LOGMODE_=292
-	,STRING_=293
-	,HEXNUMBER_=294
-	,INTNUMBER_=295
-	,DUID_=296
-	,STRICT_RFC_NO_ROUTING_=297
-	,PD_=298
-	,DUID_TYPE_=299
-	,DUID_TYPE_LLT_=300
-	,DUID_TYPE_LL_=301
-	,DUID_TYPE_EN_=302
-	,AUTH_ENABLED_=303
-	,AUTH_ACCEPT_METHODS_=304
-	,DIGEST_NONE_=305
-	,DIGEST_PLAIN_=306
-	,DIGEST_HMAC_MD5_=307
-	,DIGEST_HMAC_SHA1_=308
-	,DIGEST_HMAC_SHA224_=309
-	,DIGEST_HMAC_SHA256_=310
-	,DIGEST_HMAC_SHA384_=311
-	,DIGEST_HMAC_SHA512_=312
-	,STATELESS_=313
-	,ANON_INF_REQUEST_=314
-	,INSIST_MODE_=315
-	,INACTIVE_MODE_=316
-	,EXPERIMENTAL_=317
-	,ADDR_PARAMS_=318
-	,MAPPING_PREFIX_=319
+	,NOTIFY_SCRIPTS_=290
+	,LOGNAME_=291
+	,LOGLEVEL_=292
+	,LOGMODE_=293
+	,STRING_=294
+	,HEXNUMBER_=295
+	,INTNUMBER_=296
+	,DUID_=297
+	,STRICT_RFC_NO_ROUTING_=298
+	,SKIP_CONFIRM_=299
+	,PD_=300
+	,DUID_TYPE_=301
+	,DUID_TYPE_LLT_=302
+	,DUID_TYPE_LL_=303
+	,DUID_TYPE_EN_=304
+	,AUTH_ENABLED_=305
+	,AUTH_ACCEPT_METHODS_=306
+	,DIGEST_NONE_=307
+	,DIGEST_PLAIN_=308
+	,DIGEST_HMAC_MD5_=309
+	,DIGEST_HMAC_SHA1_=310
+	,DIGEST_HMAC_SHA224_=311
+	,DIGEST_HMAC_SHA256_=312
+	,DIGEST_HMAC_SHA384_=313
+	,DIGEST_HMAC_SHA512_=314
+	,STATELESS_=315
+	,ANON_INF_REQUEST_=316
+	,INSIST_MODE_=317
+	,INACTIVE_MODE_=318
+	,EXPERIMENTAL_=319
+	,ADDR_PARAMS_=320
+	,MAPPING_PREFIX_=321
+	,TUNNEL_MODE_=322
 
 
 #line 215 "../bison++/bison.h"
