@@ -4,9 +4,9 @@
  * authors: Tomasz Mrugalski <thomson@klub.com.pl>
  *          Marek Senderski <msend@o2.pl>
  *
- * released under GNU GPL v2 or later licence
+ * released under GNU GPL v2 only licence
  *
- * $Id: ClntIfaceIface.cpp,v 1.13 2008-02-25 17:49:07 thomson Exp $
+ * $Id: ClntIfaceIface.cpp,v 1.19 2008-11-13 21:05:42 thomson Exp $
  *
  */
 
@@ -30,6 +30,9 @@ TClntIfaceIface::TClntIfaceIface(char * name, int id, unsigned int flags, char*
     this->NTPServerLst.clear();
     this->Timezone = "";
 
+    TunnelMode = -1;
+    TunnelEndpoint = 0;
+
     unlink(WORKDIR"/"OPTION_DNS_SERVERS_FILENAME);
     unlink(WORKDIR"/"OPTION_DOMAINS_FILENAME);
     unlink(WORKDIR"/"OPTION_NTP_SERVERS_FILENAME);
@@ -43,6 +46,8 @@ TClntIfaceIface::TClntIfaceIface(char * name, int id, unsigned int flags, char*
 
     this->LifetimeTimeout = DHCPV6_INFINITY;
     this->LifetimeTimestamp = now();
+
+    setPrefixLength(CLIENT_DEFAULT_PREFIX_LENGTH);
 }
 
 /*
@@ -605,6 +610,7 @@ void TClntIfaceIface::removeAllOpts() {
     if (this->Timezone.length()) {
 	Log(Notice) << "Timezone " << this->Timezone << " unset on the interface "
 	<< this->getName() << "/" << this->getID() <<"." << LogEnd;
+	this->Timezone = "";
 	timezone_del( this->getName(), this->getID(), this->Timezone.c_str() );
     }
 
@@ -668,6 +674,25 @@ List(TIPv6Addr) TClntIfaceIface::getDNSServerLst() {
 	return DNSServerLst;
 }
 
+bool TClntIfaceIface::setTunnelMode(int mode, SPtr<TIPv6Addr> remoteEndpoint)
+{
+    Log(Debug) << "Tunnel: tunnel set to type=" << mode << ", remote end=" << remoteEndpoint->getPlain()
+	       << " on the " << getFullName() << " interface." << LogEnd;
+    TunnelMode = mode;
+    TunnelEndpoint = remoteEndpoint;
+    return true;
+}
+
+int  TClntIfaceIface::getTunnelMode()
+{
+    return TunnelMode;
+}
+
+SPtr<TIPv6Addr> TClntIfaceIface::getTunnelEndpoint()
+{
+    return TunnelEndpoint;
+}
+
 // --------------------------------------------------------------------
 // --- operators ------------------------------------------------------
 // --------------------------------------------------------------------
@@ -706,6 +731,8 @@ ostream & operator <<(ostream & strum, TClntIfaceIface &x) {
     while (sock = x.getSocket() ) {
 	strum << "    " << *sock;
     }
+    strum << "    <prefixLength>" << x.getPrefixLength() << "</prefixLength>" << endl;
+
     strum << "    <!-- options -->" << endl;
 
     // --- option: DNS-SERVERS ---
