@@ -5,14 +5,15 @@
  *          Marek Senderski <msend@o2.pl>
  * changes: Michal Kowalczuk <michal@kowalczuk.eu>
  *
- * released under GNU GPL v2 or later licence
+ * released under GNU GPL v2 only licence
  *
- * $Id: ClntIfaceMgr.cpp,v 1.44 2008-06-18 20:18:03 thomson Exp $
+ * $Id: ClntIfaceMgr.cpp,v 1.50 2008-08-30 21:41:12 thomson Exp $
  */
 
 #include "Portable.h"
 #include "SmartPtr.h"
 #include "ClntIfaceMgr.h"
+#include "ClntAddrMgr.h"
 #include "Msg.h"
 #include "ClntMsgReply.h"
 #include "ClntMsgAdvertise.h"
@@ -407,7 +408,7 @@ bool TClntIfaceMgr::modifyPrefix(int iface, SPtr<TIPv6Addr> prefix, int prefixLe
 	switch (mode) {
 	case PREFIX_MODIFY_ADD:
 	{
-	    sprintf(buf, "%s %s", cmd1, prefix->getPlain());
+	    sprintf(buf, "sh %s %s", cmd1, prefix->getPlain());
 	    Log(Notice) << "Executing external command to ADD prefix: " << buf << LogEnd;
 	    returnCode = system(buf);
 	    Log(Notice) << "ReturnCode = " << returnCode << LogEnd;
@@ -415,8 +416,9 @@ bool TClntIfaceMgr::modifyPrefix(int iface, SPtr<TIPv6Addr> prefix, int prefixLe
 	}
 	case PREFIX_MODIFY_DEL:
 	{
-	    sprintf(buf, "%s %s", cmd2, prefix->getPlain());
+	    sprintf(buf, "sh %s %s", cmd2, prefix->getPlain());
 	    Log(Notice) << "Executing external command to DEL prefix: " << buf << LogEnd;
+
 	    returnCode = system(buf);
 	    Log(Notice) << "ReturnCode = " << returnCode << LogEnd;
 	    break;
@@ -578,6 +580,91 @@ void TClntIfaceMgr::redetectIfaces() {
     if_list_release(ifaceList); // allocated in pure C, and so release it there
 }
 
+void TClntIfaceMgr::notifyScripts(int msgType, int ifindex)
+{
+    if (!ClntCfgMgr->getNotifyScripts()) {
+	Log(Debug) << "Not executing external script (Notify script disabled)." << LogEnd;
+	return;
+    }
+
+    PrefixModifyMode mode;
+    string action;
+
+    switch (msgType)
+    {
+    case REQUEST_MSG:
+	mode = PREFIX_MODIFY_ADD;
+	action = "add";
+	break;
+    case RELEASE_MSG:
+	mode = PREFIX_MODIFY_DEL;
+	action = "delete";
+	break;
+    case RENEW_MSG:
+    default:
+	mode = PREFIX_MODIFY_UPDATE;
+	action = "update";
+    }
+
+    SPtr<TClntIfaceIface> iface = (Ptr*)getIfaceByID(ifindex);
+    if (!iface) {
+	Log(Error) << "Unable to find interface with ifindex=" << ifindex << ". Notification NOT sent." << LogEnd;
+	return;
+    }
+
+    int tunnelMode = iface->getTunnelMode();
+    SPtr<TIPv6Addr> ip;
+    SPtr<TIPv6Addr> remoteEndpoint;
+    SPtr<TAddrPrefix> prefix;
+
+    if (tunnelMode!=0)
+    {
+	remoteEndpoint = iface->getTunnelEndpoint();
+    }
+
+    ClntAddrMgr->firstIA();
+    SPtr<TAddrIA> ia = ClntAddrMgr->getIA();
+    if (!ia)
+    {
+	Log(Error) << "Unable to find any IAs defined in the AddrMgr." << LogEnd;
+	return;
+    }
+
+    ia->firstAddr();
+    if (ia->countAddr())
+    {
+	SPtr<TAddrAddr> addr = ia->getAddr();
+	ip = addr->get();
+    }
+
+    ClntAddrMgr->firstPD();
+    ia = ClntAddrMgr->getPD();
+    if (ia)
+    {
+	ia->firstPrefix();
+	prefix = ia->getPrefix();
+    }
+
+    if (!ip)
+	ip = new TIPv6Addr("::", true);
+
+    if (!remoteEndpoint)
+	remoteEndpoint = new TIPv6Addr("::", true);
+    
+    if (!prefix) 
+	prefix = new TAddrPrefix(new TIPv6Addr("::", true), 0, 0, 0);
+
+    stringstream tmp;
+    tmp << "sh ./notify " << " " << ip->getPlain() << " "
+	<< prefix->get()->getPlain() << " " << prefix->getLength() << " "
+	<< remoteEndpoint->getPlain() << " " << tunnelMode << " " << action;
+    Log(Info) << "About to execute command: " << tmp.str() << LogEnd;
+
+    int returnCode = system(tmp.str().c_str());
+
+    Log(Info) << "Return code=" << returnCode << LogEnd;
+}
+
 ostream & operator <<(ostream & strum, TClntIfaceMgr &x) {
     strum << "<ClntIfaceMgr>" << std::endl;
     SmartPtr<TClntIfaceIface> ptr;
