@@ -5,9 +5,9 @@
  *          Marek Senderski <msend@o2.pl>
  * changes: Michal Kowalczuk <michal@kowalczuk.eu>
  *
- * released under GNU GPL v2 or later licence
+ * released under GNU GPL v2 only licence
  *
- * $Id: ClntMsg.cpp,v 1.36 2008-06-02 00:15:00 thomson Exp $
+ * $Id: ClntMsg.cpp,v 1.42 2008-11-13 22:40:26 thomson Exp $
  */
 
 #ifdef WIN32
@@ -188,7 +188,7 @@ TClntMsg::TClntMsg(SmartPtr<TClntIfaceMgr> IfaceMgr,
 	case OPTION_DOMAIN_LIST:
 	    ptr = new TClntOptDomainName(buf+pos, length, this);
 	    break;
-	case OPTION_TIME_ZONE:
+	case OPTION_NEW_TZDB_TIMEZONE:
 	    ptr = new TClntOptTimeZone(buf+pos, length,this);
 	    break;
 	case OPTION_SIP_SERVER_A:
@@ -329,13 +329,14 @@ unsigned long TClntMsg::getTimeout()
 
 void TClntMsg::send()
 {
+    srand(now());
     if (!RC)
-        RT=(int)floor(0.5+IRT+IRT*(0.2*(double)rand()/(double)RAND_MAX-0.1));
+        RT=(int)(0.5+IRT+IRT*(0.2*(double)rand()/(double)RAND_MAX-0.1));
     else
-	RT =(int) floor(0.5+2.0*RT+RT*(0.2*(double)rand()/(double)RAND_MAX-0.1));
+	RT =(int)(0.5+2.0*RT+RT*(0.2*(double)rand()/(double)RAND_MAX-0.1));
 
     if (MRT != 0 && RT>MRT) 
-	RT = (int) floor(0.5+MRT + MRT*(0.2*(double)rand()/(double)RAND_MAX-0.1));
+	RT = (int)(0.5+MRT + MRT*(0.2*(double)rand()/(double)RAND_MAX-0.1));
     
     if ((MRD != 0) && (RT>MRD))
         RT = MRD;
@@ -442,6 +443,14 @@ void TClntMsg::appendAuthenticationOption(SmartPtr<TClntAddrMgr> AddrMgr)
 #endif
 }
 
+void TClntMsg::appendElapsedOption() {
+    // include ELAPSED option
+
+    if (!getOption(OPTION_ELAPSED_TIME))
+	Options.append(new TClntOptElapsed(this));
+}
+
+/* CHANGED in this function: According to RFC3315,'status==STATE_NOTCONFIGURED' is not a must.*/
 /*
  * this method adds requested (which have status==STATE_NOTCONFIGURED) options
  */
@@ -460,14 +469,14 @@ void TClntMsg::appendRequestedOptions() {
 	optORO->addOption(OPTION_UNICAST);
 	Log(Debug) << "Adding UNICAST to ORO." << LogEnd;
     }
-
+ 
     if (ClntCfgMgr->addInfRefreshTime()) {
 	optORO->addOption(OPTION_INFORMATION_REFRESH_TIME);
-	Log(Debug) << "#### Adding INFORMATION REFRESH TIME to ORO." << LogEnd;
+	Log(Debug) << "Adding INFORMATION REFRESH TIME to ORO." << LogEnd;
     }
 
     // --- option: DNS-SERVERS ---
-    if ( iface->isReqDNSServer() && (iface->getDNSServerState()==STATE_NOTCONFIGURED) ) {
+    if ( iface->isReqDNSServer() ) {
 	optORO->addOption(OPTION_DNS_SERVERS);
 	
 	List(TIPv6Addr) * dnsLst = iface->getProposedDNSServerLst();
@@ -480,7 +489,7 @@ void TClntMsg::appendRequestedOptions() {
     }
 
     // --- option: DOMAINS --
-    if ( iface->isReqDomain() && (iface->getDomainState()==STATE_NOTCONFIGURED) ) {
+    if ( iface->isReqDomain() ) {
 	optORO->addOption(OPTION_DOMAIN_LIST);
 
 	List(string) * domainsLst = iface->getProposedDomainLst();
@@ -493,7 +502,7 @@ void TClntMsg::appendRequestedOptions() {
     }
 
     // --- option: NTP SERVER ---
-    if ( iface->isReqNTPServer() && (iface->getNTPServerState()==STATE_NOTCONFIGURED) ) {
+    if ( iface->isReqNTPServer() ) {
 	optORO->addOption(OPTION_SNTP_SERVERS);
 
 	List(TIPv6Addr) * ntpLst = iface->getProposedNTPServerLst();
@@ -506,8 +515,8 @@ void TClntMsg::appendRequestedOptions() {
     }
         
     // --- option: TIMEZONE ---
-    if ( iface->isReqTimezone() && (iface->getTimezoneState()==STATE_NOTCONFIGURED) ) {
-	optORO->addOption(OPTION_TIME_ZONE);
+    if ( iface->isReqTimezone() ) {
+	optORO->addOption(OPTION_NEW_TZDB_TIMEZONE);
 
 	string timezone = iface->getProposedTimezone();
 	if (timezone.length()) {
@@ -519,7 +528,7 @@ void TClntMsg::appendRequestedOptions() {
     }
 
     // --- option: SIP-SERVERS ---
-    if ( iface->isReqSIPServer() && (iface->getSIPServerState()==STATE_NOTCONFIGURED) ) {
+    if ( iface->isReqSIPServer() ) {
 	optORO->addOption(OPTION_SIP_SERVER_A);
 	
 	List(TIPv6Addr) * lst = iface->getProposedSIPServerLst();
@@ -532,7 +541,7 @@ void TClntMsg::appendRequestedOptions() {
     }
 
     // --- option: SIP-DOMAINS ---
-    if ( iface->isReqSIPDomain() && (iface->getSIPDomainState()==STATE_NOTCONFIGURED) ) {
+    if ( iface->isReqSIPDomain() ) {
 	optORO->addOption(OPTION_SIP_SERVER_D);
 
 	List(string) * domainsLst = iface->getProposedSIPDomainLst();
@@ -545,7 +554,7 @@ void TClntMsg::appendRequestedOptions() {
     }
 
     // --- option: FQDN ---
-    if ( iface->isReqFQDN() && (iface->getFQDNState()==STATE_NOTCONFIGURED) ) {
+    if ( iface->isReqFQDN() ) {
 	optORO->addOption(OPTION_FQDN);
 
 	string fqdn = iface->getProposedFQDN();
@@ -558,7 +567,7 @@ void TClntMsg::appendRequestedOptions() {
     }
 
     // --- option: NIS-SERVERS ---
-    if ( iface->isReqNISServer() && (iface->getNISServerState()==STATE_NOTCONFIGURED) ) {
+    if ( iface->isReqNISServer() ) {
 	optORO->addOption(OPTION_NIS_SERVERS);
 	
 	List(TIPv6Addr) * lst = iface->getProposedNISServerLst();
@@ -571,7 +580,7 @@ void TClntMsg::appendRequestedOptions() {
     }
 
     // --- option: NIS-DOMAIN ---
-    if ( iface->isReqNISDomain() && (iface->getNISDomainState()==STATE_NOTCONFIGURED) ) {
+    if ( iface->isReqNISDomain() ) {
 	optORO->addOption(OPTION_NIS_DOMAIN_NAME);
 	string domain = iface->getProposedNISDomain();
 	if (domain.length()) {
@@ -582,7 +591,7 @@ void TClntMsg::appendRequestedOptions() {
     }
 
     // --- option: NIS+-SERVERS ---
-    if ( iface->isReqNISPServer() && (iface->getNISPServerState()==STATE_NOTCONFIGURED) ) {
+    if ( iface->isReqNISPServer() ) {
 	optORO->addOption(OPTION_NISP_SERVERS);
 	
 	List(TIPv6Addr) * lst = iface->getProposedNISPServerLst();
@@ -595,7 +604,7 @@ void TClntMsg::appendRequestedOptions() {
     }
 
     // --- option: NIS+-DOMAIN ---
-    if ( iface->isReqNISPDomain() && (iface->getNISPDomainState()==STATE_NOTCONFIGURED) ) {
+    if ( iface->isReqNISPDomain() ) {
 	optORO->addOption(OPTION_NISP_DOMAIN_NAME);
 	string domain = iface->getProposedNISPDomain();
 	if (domain.length()) {
@@ -614,7 +623,7 @@ void TClntMsg::appendRequestedOptions() {
 	optORO->addOption(OPTION_INFORMATION_REFRESH_TIME);
 
     // --- option: VENDOR-SPEC ---
-    if ( iface->isReqVendorSpec() && (iface->getVendorSpecState()==STATE_NOTCONFIGURED) ) {
+    if ( iface->isReqVendorSpec() ) {
 	optORO->addOption(OPTION_VENDOR_OPTS);
 	iface->setVendorSpecState(STATE_INPROCESS);
 
@@ -636,8 +645,7 @@ void TClntMsg::appendRequestedOptions() {
     if (addrParams)
 	optORO->addOption(OPTION_ADDRPARAMS);
 
-    // include ELAPSED option
-    Options.append(new TClntOptElapsed(this));
+    appendElapsedOption();
 
 #ifndef MOD_DISABLE_AUTH
     if (this->MsgType == SOLICIT_MSG) {
