@@ -6,9 +6,9 @@
  * changes: Krzysztof Wnuk <keczi@poczta.onet.pl>
  *          Michal Kowalczuk <michal@kowalczuk.eu>
  *
- * released under GNU GPL v2 or later licence
+ * released under GNU GPL v2 only licence
  *
- * $Id: ClntMsgRenew.cpp,v 1.17 2008-03-06 21:36:57 thomson Exp $
+ * $Id: ClntMsgRenew.cpp,v 1.22 2008-08-29 00:07:28 thomson Exp $
  *
  */
 
@@ -46,7 +46,7 @@ TClntMsgRenew::TClntMsgRenew(SmartPtr<TClntIfaceMgr> IfaceMgr,
     }
 
     // retransmit until T2 has been reached or any address has expired
-    //it shhould be the same for all IAs
+    //it should be the same for all IAs
     unsigned int timeout = DHCPV6_INFINITY;
 
     SPtr<TAddrIA> ia;
@@ -95,6 +95,7 @@ TClntMsgRenew::TClntMsgRenew(SmartPtr<TClntIfaceMgr> IfaceMgr,
 	Options.append(new TClntOptIA_PD(ia, this));
     }
 
+    appendRequestedOptions();
     appendAuthenticationOption(AddrMgr);
 
     pkt = new char[getSize()];
@@ -164,12 +165,21 @@ void TClntMsgRenew::answer(SmartPtr<TClntMsg> Reply)
     //There is no use to send Rebind even if server realesed some addresses/IAs
     //in such a case new Solicit message should be sent
     IsDone = true;
+
+    ClntIfaceMgr->notifyScripts(RENEW_MSG, Iface);
 }
 
+/** 
+ * @brief changes IA state to not cofigured.
+ * 
+ * @param IAID 
+ */
 void TClntMsgRenew::releaseIA(long IAID)
 {
-    // setStatus(NOTCONFIGURED);
-    //FIXME:
+    SPtr<TAddrIA> ia = getClntAddrMgr()->getIA(IAID);
+    if(ia){
+        ia->setState(STATE_NOTCONFIGURED);
+    }  
 }
 
 void TClntMsgRenew::doDuties()
