@@ -1,14 +1,17 @@
 /*
  * Dibbler - a portable DHCPv6
  *
- * authors: Tomasz Mrugalski <thomson@klub.com.pl>
- *          Marek Senderski <msend@o2.pl>
+ * author: Krzysztof Wnuk <keczi@poczta.onet.pl>
  *
- * released under GNU GPL v2 or later licence
- *
- * $Id: ClntOptFQDN.h,v 1.1 2004/10/25 20:45:53 thomson Exp $
+ * $Id: ClntOptFQDN.h,v 1.3 2006-10-06 00:42:13 thomson Exp $
  *
  * $Log: ClntOptFQDN.h,v $
+ * Revision 1.3  2006-10-06 00:42:13  thomson
+ * Initial PD support.
+ *
+ * Revision 1.2  2006-03-02 00:59:22  thomson
+ * ClntOptFQDN implemented for real.
+ *
  * Revision 1.1  2004/10/25 20:45:53  thomson
  * Option support, parsers rewritten. ClntIfaceMgr now handles options.
  *
@@ -18,11 +21,15 @@
 #ifndef CLNTOPTFQDN_H
 #define CLNTOPTFQDN_H
 
-#include "OptString.h"
+#include "OptFQDN.h"
 #include "DUID.h"
 #include "SmartPtr.h"
+#include "ClntIfaceMgr.h"
 
-class TClntOptFQDN : public TOptString
+
+// void *updateDNS(void *IfaceMgr);
+
+class TClntOptFQDN : public TOptFQDN
 {
  public:
     TClntOptFQDN(string fqdn, TMsg* parent);
