@@ -6,9 +6,12 @@
  *
  * released under GNU GPL v2 licence
  *
- * $Id: ClntOptTimeZone.cpp,v 1.7 2007-01-07 20:18:46 thomson Exp $
+ * $Id: ClntOptTimeZone.cpp,v 1.8 2008-11-13 22:40:26 thomson Exp $
  *
  * $Log: ClntOptTimeZone.cpp,v $
+ * Revision 1.8  2008-11-13 22:40:26  thomson
+ * New timezones implemented: RFC4833 (by Petr Pisar) (bug #185)
+ *
  * Revision 1.7  2007-01-07 20:18:46  thomson
  * State enum names changed.
  *
@@ -33,12 +36,12 @@
 #include "Logger.h"
 
 TClntOptTimeZone::TClntOptTimeZone(string domain, TMsg* parent)
-    :TOptString(OPTION_TIME_ZONE, domain, parent) {
+    :TOptString(OPTION_NEW_TZDB_TIMEZONE, domain, parent) {
 
 }
 
 TClntOptTimeZone::TClntOptTimeZone(char *buf, int bufsize, TMsg* parent)
-    :TOptString(OPTION_TIME_ZONE, buf,bufsize, parent) {
+    :TOptString(OPTION_NEW_TZDB_TIMEZONE, buf,bufsize, parent) {
     // FIXME: do some validity check
 
 }
