@@ -6,9 +6,9 @@
  * changes: Krzysztof Wnuk <keczi@poczta.onet.pl>
  *          Michal Kowalczuk <michal@kowalczuk.eu>
  *
- * released under GNU GPL v2 or later licence
+ * released under GNU GPL v2 only licence
  *
- * $Id: ClntTransMgr.cpp,v 1.58 2008-03-23 13:11:34 thomson Exp $
+ * $Id: ClntTransMgr.cpp,v 1.62 2008-11-13 22:18:17 thomson Exp $
  *
  */
 
@@ -50,16 +50,16 @@ TClntTransMgr::TClntTransMgr(SmartPtr<TClntIfaceMgr> ifaceMgr,
 {
     // should we set REUSE option during binding sockets?
 #ifdef MOD_CLNT_BIND_REUSE
-    this->BindReuse = true;
+    BindReuse = true;
 #else
-    this->BindReuse = false;
+    BindReuse = false;
 #endif
 
-    this->IsDone = true;
+    IsDone = true;
 
-    this->IfaceMgr = ifaceMgr;
-    this->AddrMgr  = addrMgr;
-    this->CfgMgr   = cfgMgr;
+    IfaceMgr = ifaceMgr;
+    AddrMgr  = addrMgr;
+    CfgMgr   = cfgMgr;
 
     if (this->BindReuse)
 	Log(Debug) << "Bind reuse enabled (multiple instances allowed)." << LogEnd;
@@ -78,11 +78,7 @@ TClntTransMgr::TClntTransMgr(SmartPtr<TClntIfaceMgr> ifaceMgr,
 	}
     }
 
-    // FIXME: CONFIRM support is not complete
-    // this->checkDB();
-    // checkConfirm();
-    this->ConfirmEnabled=true;
-
+    this->checkDB();
     this->Shutdown = false;
     this->IsDone = false;
 }
@@ -210,21 +206,29 @@ void TClntTransMgr::removeExpired() {
     }
 }
 
+/** 
+ * checks if loaded Address database is sanite (i.e. does not reffer to non-existing interface)
+ * 
+ */
 void TClntTransMgr::checkDB()
 {
+    SmartPtr<TClntCfgIface> iface;
     SmartPtr <TAddrIA> ptrIA;
-    AddrMgr->firstIA();
+    SmartPtr<TAddrAddr> ptrAddr;
+
     AddrMgr->doDuties();
+    AddrMgr->firstIA();
     while ( ptrIA = AddrMgr->getIA()) {
-        SmartPtr<TIfaceIface> ptrIface;
-        ptrIface = IfaceMgr->getIfaceByID( ptrIA->getIface() );
-        if (!ptrIface) {
+        iface = CfgMgr->getIface( ptrIA->getIface() );
+        if (!iface) {
+	    Log(Warning) << "IA (iaid=" << ptrIA->getIAID() << ") loaded from old file, but currently there is no iface with ifindex="
+			 << ptrIA->getIface();
             // IA with non-existent iface, purge iface
-            SmartPtr<TAddrAddr> ptrAddr;
             ptrIA->firstAddr();
             while (ptrAddr = ptrIA->getAddr())
                 ptrIA->delAddr( ptrAddr->get() );
             ptrIA->setState(STATE_NOTCONFIGURED);
+	    AddrMgr->delIA(ptrIA->getIAID());
         }
     }
 }
@@ -309,6 +313,11 @@ void TClntTransMgr::doDuties()
     this->CfgMgr->dump();
 
     if (!this->Shutdown && !this->IsDone) {
+
+        // did we switched links lately?
+	// are there any IAs to confirm?
+	checkConfirm();
+	
         // are there any tentative addrs?
         checkDecline();
 
@@ -322,11 +331,6 @@ void TClntTransMgr::doDuties()
         // are there any aging IAs or PDs?
         checkRenew();
 
-        // did we switched links lately?
-	// FIXME: for now, we cannot check link switches, so checkConfirm
-	// is run only during startup
-        //checkConfirm();        
-
         //Maybe we require only infromations concernig link
         checkInfRequest();
     } 
@@ -352,7 +356,7 @@ void TClntTransMgr::shutdown()
     this->Shutdown = true;
     Log(Notice) << "Shutting down entire client." << LogEnd;
 	
-    // delete all weird-state and address-free IAs 
+    // delete all weird-state/innormal-state and address-free IAs 
     AddrMgr->firstIA();
     while (ptrFirstIA = AddrMgr->getIA()) {
         if ( (ptrFirstIA->getState() != STATE_CONFIGURED && ptrFirstIA->getState() != STATE_INPROCESS) ||
@@ -389,10 +393,10 @@ void TClntTransMgr::shutdown()
                     }
                     ptrNextIA->firstAddr();
 
-		    // addrs are RELEASEd in ClntMsgRelease.cpp
+		    // addresses are deleted in ClntMsgRelease.cpp, DO NOT delete them here
                     // while (ptrAddr = ptrNextIA->getAddr() ) {
-                    //     ptrIface->delAddr( ptrAddr->get() );
-                    //}
+                    //    ptrIface->delAddr( ptrAddr->get(),ptrAddr->getPrefix() );
+                    // }
 
                     // delete IA from AddrMgr
                     AddrMgr->delIA( ptrNextIA->getIAID() );
@@ -457,8 +461,9 @@ void TClntTransMgr::shutdown()
     if (releasedPDs.count())
 	this->sendRelease(releasedIAs, 0, releasedPDs);
     
-    //doDuties(); // just to send RELEASE msg
-    //Transactions.clear(); // delete all transactions
+    //CHANGED:the following two lines are uncommented.
+    doDuties(); // just to send RELEASE msg
+    Transactions.clear(); // delete all transactions
 
     // clean up options
     this->IfaceMgr->removeAllOpts();
@@ -496,8 +501,14 @@ void TClntTransMgr::relayMsg(SmartPtr<TClntMsg>  msgAnswer)
     }
 
     if (!found) 
-        Log(Warning) << "Message with wrong transID (" << hex << msgAnswer->getTransID() << dec
-		     << ") received. Ignoring." << LogEnd;
+    {
+	if (!Shutdown)
+	    Log(Warning) << "Message with wrong transID (0x" << hex << msgAnswer->getTransID() << dec
+			 << ") received. Ignoring." << LogEnd;
+	else
+	    Log(Debug) << "Message with transID=0x" << hex << msgAnswer->getTransID() << dec
+		       << " received, but ignored during shutdown." << LogEnd;
+    } 
     CfgMgr->dump();
     AddrMgr->dump();
 }
@@ -701,28 +712,36 @@ void TClntTransMgr::checkSolicit() {
 
 void TClntTransMgr::checkConfirm()
 {
-    //FIXME: When should we send CONFIRM? How to detect switching to new link?
-    //Is it a start of address of manager
-    if(!ConfirmEnabled)
-        return;
-    SmartPtr<TAddrIA> ptrIA;
-    SmartPtr<TIfaceIface> ptrIface;
-    IfaceMgr->firstIface();
-    while(ptrIface=IfaceMgr->getIface())
+    SPtr<TAddrIA> ptrIA;
+    SPtr<TClntCfgIface> iface;
+    SPtr<TClntCfgIA> cfgIA;
+    List(TAddrIA) IALst;
+
+    CfgMgr->firstIface();
+    while(iface=CfgMgr->getIface())
     {
-        TContainer<SmartPtr<TAddrIA> > IALst;
-        AddrMgr->firstIA();
-        while(ptrIA=AddrMgr->getIA())
-        {
-            if(ptrIA->getIface()==ptrIface->getID())
-            {
-                IALst.append(ptrIA);
-                ptrIA->setState(STATE_INPROCESS);
-            }
-        }
-        if (IALst.count())
+	IALst.clear();
+	iface->firstIA();
+	while (cfgIA = iface->getIA()) {
+	    AddrMgr->firstIA();
+	    while(ptrIA=AddrMgr->getIA())
+	    {
+		if (ptrIA->getState()!=STATE_CONFIRMME)	
+		    continue;
+		if (ptrIA->getIface()==iface->getID())
+		{
+		    IALst.append(ptrIA);
+		    ptrIA->setState(STATE_INPROCESS);
+		    cfgIA->setState(STATE_INPROCESS);
+		}
+	    }
+	}	    
+
+        if (IALst.count()) {
+	    Log(Info) << "Creating CONFIRM: " << IALst.count() << " IA(s) on " << iface->getFullName() << LogEnd;
             Transactions.append(
-            new TClntMsgConfirm(IfaceMgr,That,CfgMgr,AddrMgr,ptrIface->getID(),IALst));
+		new TClntMsgConfirm(IfaceMgr,That,CfgMgr,AddrMgr,iface->getID(),IALst));
+	}
     }
 }
 
@@ -801,6 +820,8 @@ void TClntTransMgr::checkRenew()
     SmartPtr<TAddrIA> ia;
     SmartPtr<TAddrIA> iaPattern;
     AddrMgr->firstIA();
+
+    // Need to be fixed:?? how to deal with mutiple network interfaces.
     while (ia = AddrMgr->getIA() ) 
     {
         if ( (ia->getT1Timeout()!=0) || 
@@ -847,8 +868,7 @@ void TClntTransMgr::checkRenew()
     }
 
     if (iaLst.count() + pdLst.count() == 0) {
-	// FIXME: something is wrong
-	Log(Error) << "RENEW send attempted, but there are no IA(s) or PD(s) to send." << LogEnd;
+	// there are no IAs or PD to refresh. Just do nothing.
 	return;
     }
 	 
