@@ -4,9 +4,9 @@
  * authors: Tomasz Mrugalski <thomson@klub.com.pl>                           
  *          Marek Senderski <msend@o2.pl>                                    
  *                                                                           
- * released under GNU GPL v2 or later licence                                
+ * released under GNU GPL v2 only licence                                
  *                                                                           
- * $Id: DHCPClient.cpp,v 1.28 2007-04-01 04:53:19 thomson Exp $
+ * $Id: DHCPClient.cpp,v 1.33 2008-11-13 22:18:17 thomson Exp $
  *                                                                           
  */
 
@@ -22,10 +22,12 @@
 #include "Portable.h"
 
 volatile int serviceShutdown;
+volatile int linkstateChange;
 
 TDHCPClient::TDHCPClient(string config)
 {
     serviceShutdown = 0;
+    linkstateChange = 0;
     srand(now());
     this->IsDone = false;
 
@@ -57,12 +59,42 @@ TDHCPClient::TDHCPClient(string config)
 	return;
     }
 
+    if (CfgMgr->useConfirm())
+	initLinkStateChange();
+
     TransMgr->setContext(TransMgr);
 }
 
+/** 
+ * initializes low-level link state change detection mechanism
+ * 
+ */
+void TDHCPClient::initLinkStateChange()
+{
+    Log(Debug) << "LinkState change detection not fully supported (disabled for now)." << LogEnd;
+    return; // disable this for now
+
+    memset((void*)&this->linkstates, 0, sizeof(linkstates));
+
+    CfgMgr->firstIface();
+    SPtr<TClntCfgIface> iface;
+    Log(Debug) << "Initialising link-state detection for interfaces: ";
+    while (iface = CfgMgr->getIface()) 
+    {
+	linkstates.ifindex[linkstates.cnt++] = iface->getID();
+	Log(Cont) << iface->getID() << " ";
+    }
+    Log(Cont) << LogEnd;
+    
+    link_state_change_init(&linkstates, &linkstateChange);
+}
+
 void TDHCPClient::stop() {
     serviceShutdown = 1;
 
+    if (CfgMgr->useConfirm())
+	link_state_change_cleanup();
+
 #ifdef WIN32
     // just to break select() in WIN32 systems
     SmartPtr<TIfaceIface> iface = IfaceMgr->getIfaceByID(TransMgr->getCtrlIface());
@@ -75,6 +107,21 @@ void TDHCPClient::stop() {
 #endif
 }
 
+/* Function removed. Please use link_state_changed() instead */
+/* void TDHCPClient::changeLinkstate(int iface_id) */
+
+void TDHCPClient::resetLinkstate() {
+    linkstates.cnt = 0;
+    for (int i = 0; i<MAX_LINK_STATE_CHANGES_AT_ONCE; i++)
+	linkstates.ifindex[i]=0; // ugly, but safe way of zeroing table
+    linkstateChange = 0;
+}
+
+char* TDHCPClient::getCtrlIface(){
+    SmartPtr<TIfaceIface> iface = IfaceMgr->getIfaceByID(TransMgr->getCtrlIface());
+    return iface->getName();
+;
+}
 
 void TDHCPClient::run()
 {
@@ -84,6 +131,11 @@ void TDHCPClient::run()
 	if (serviceShutdown)
 	    TransMgr->shutdown();
 	
+        if (linkstateChange) {
+	    AddrMgr->setIA2Confirm(&linkstates);
+	    this->resetLinkstate();
+        }
+
 	TransMgr->doDuties();
 	
 	unsigned int timeout = TransMgr->getTimeout();
@@ -129,6 +181,14 @@ void TDHCPClient::setWorkdir(std::string workdir) {
     }
 }
 
+SmartPtr<TClntAddrMgr> TDHCPClient::getAddrMgr() {
+    return this->AddrMgr;
+}
+
+SmartPtr<TClntCfgMgr> TDHCPClient::getCfgMgr() {
+    return this->CfgMgr;
+}
+
 TDHCPClient::~TDHCPClient()
 {
     if (TransMgr)
