@@ -4,9 +4,15 @@
  * authors: Tomasz Mrugalski <thomson@klub.com.pl>
  *          Marek Senderski <msend@o2.pl>
  *
- * $Id: DUID.h,v 1.6 2006/01/12 00:23:35 thomson Exp $
+ * $Id: DUID.h,v 1.8 2006-11-03 20:07:06 thomson Exp $
  *
  * $Log: DUID.h,v $
+ * Revision 1.8  2006-11-03 20:07:06  thomson
+ * Server-side vendorSpecInfo support added.
+ *
+ * Revision 1.7  2006-08-21 21:03:44  thomson
+ * Load procedure fixed.
+ *
  * Revision 1.6  2006/01/12 00:23:35  thomson
  * Cleanup changes. Now -pedantic option works.
  *
@@ -37,8 +43,8 @@ class TDUID
     friend std::ostream& operator<<(std::ostream& out,TDUID &range);
  public:
     TDUID();
-    TDUID(char* DUID,int DUIDlen); // packed
-    TDUID(char* Plain); // plain
+    TDUID(const char* DUID,int DUIDlen); // packed
+    TDUID(const char* Plain); // plain
     TDUID(const TDUID &duid);
     TDUID& operator=(const TDUID& duid);
     bool operator==(const TDUID &duid);
@@ -46,6 +52,7 @@ class TDUID
     int getLen();
     char * storeSelf(char* buf);
     const string getPlain();
+    const char * get();
 
     ~TDUID();
 
