@@ -6,9 +6,12 @@
  *
  * released under GNU GPL v2 licence
  *
- * $Id: OptStringLst.cpp,v 1.10 2007-08-26 10:26:19 thomson Exp $
+ * $Id: OptStringLst.cpp,v 1.11 2007-12-25 08:11:57 thomson Exp $
  *
  * $Log: OptStringLst.cpp,v $
+ * Revision 1.11  2007-12-25 08:11:57  thomson
+ * Several fixes after 2nd bakeoff meeting.
+ *
  * Revision 1.10  2007-08-26 10:26:19  thomson
  * Possible segfault in REBIND processing,
  * gcc 4.3.0 conformance
@@ -132,7 +135,7 @@ char * TOptStringLst::storeSelf(char* buf)
 
         *buf = cp.length(); // length of the string
         buf++;
-        memcpy(buf, cp.c_str(), x->length());
+        memcpy(buf, cp.c_str(), cp.length());
 	buf+= cp.length();
 	*buf=0;
 	buf++;
