@@ -9,12 +9,14 @@
  * Authors: Alexey Kuznetsov, <kuznet@ms2.inr.ac.ru>
  * Changes: Laszlo Valko <valko@linux.karinthy.hu>
  *
- * released under GNU GPL v2 or later licence
+ * released under GNU GPL v2 only licence
  *
- * $Id: lowlevel-linux.c,v 1.11 2008-06-22 11:43:18 thomson Exp $
+ * $Id: lowlevel-linux.c,v 1.13 2009-03-09 22:27:23 thomson Exp $
  *
  */
 
+#define _GNU_SOURCE
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -165,7 +167,7 @@ struct iface * if_list_get()
 #endif
 
 	tmp = malloc(sizeof(struct iface));
-	snprintf(tmp->name,MAX_IFNAME_LENGTH,(char*)RTA_DATA(tb[IFLA_IFNAME]));
+	snprintf(tmp->name,MAX_IFNAME_LENGTH,"%s",(char*)RTA_DATA(tb[IFLA_IFNAME]));
 	tmp->id=ifi->ifi_index;
 	tmp->flags=ifi->ifi_flags;
 	tmp->hardwareType = ifi->ifi_type;
