@@ -6,7 +6,7 @@
  *
  * released under GNU GPL v2 licence
  *
- * $Id: lowlevel-win32.c,v 1.16 2007-03-04 21:48:55 thomson Exp $
+ * $Id: lowlevel-win32.c,v 1.22 2008-10-13 22:41:18 thomson Exp $
  *
  */
 
@@ -21,6 +21,10 @@
 #include <process.h>
 #include <errno.h>
 #include <stdlib.h>
+#include <stdio.h>
+#include <fcntl.h>
+#include <sys/stat.h>
+#include <io.h>
 
 #include "Portable.h"
 #include "DHCPConst.h"
@@ -60,6 +64,80 @@ int lowlevelInit()
     return 1;
 }
 
+/* when updating this file, remember to also update copy in Port-winnt2k/lowlevel-winnt2k.c */
+uint32_t getAAASPIfromFile() {
+    char filename[1024];
+    struct stat st;
+    uint32_t ret;
+    FILE *file;
+
+    strcpy(filename, "AAA-SPI");
+
+    if (stat(filename, &st))
+        return 0;
+
+    file = fopen(filename, "r");
+    if (!file)
+        return 0;
+
+    fscanf(file, "%x", &ret);
+    fclose(file);
+
+    return ret;
+}
+
+/* when updating this file, remember to also update copy in Port-winnt2k/lowlevel-winnt2k.c */
+char * getAAAKeyFilename(uint32_t SPI)
+{
+    static char filename[1024];
+    if (SPI)
+        snprintf(filename, 1024, "%s%s%x", "", "AAA-key-", SPI);
+    else
+        strcpy(filename, "AAA-key");
+    return filename;
+}
+
+/* when updating this file, remember to also update copy in Port-winnt2k/lowlevel-winnt2k.c */
+char * getAAAKey(uint32_t SPI, uint32_t *len) {
+
+    char * filename;
+    struct stat st;
+    char * retval;
+    int offset = 0;
+    int fd;
+    int ret;
+
+    filename = getAAAKeyFilename(SPI);
+
+    if (stat(filename, &st))
+        return NULL;
+
+    fd = open(filename, O_RDONLY);
+    if (0 > fd)
+        return NULL;
+
+    /* FIXME should be freed somewhere */
+    retval = malloc(st.st_size);
+    if (!retval)
+        return NULL;
+
+    while (offset < st.st_size) {
+        ret = read(fd, retval + offset, st.st_size - offset);
+        if (!ret) break;
+        if (ret < 0) {
+            return NULL;
+        }
+        offset += ret;
+    }
+    close(fd);
+
+    if (offset != st.st_size)
+        return NULL;
+
+    *len = st.st_size;
+    return retval;
+}
+
 char * error_message()
 {
 	return Message;
@@ -393,7 +471,7 @@ int sock_recv(int fd, char * myPlainAddr, char * peerPlainAddr, char * buf, int
 
 extern int dns_add(const char* ifname, int ifaceid, const char* addrPlain) {
     
-    // netsh interface ipv6 add dns interface="eth0" address=2000::123
+    // netsh interface ipv6 add dns "eth0" address=2000::123
     char arg1[]="interface";
     char arg2[]="ipv6";
     char arg3[]="add";
@@ -402,7 +480,7 @@ extern int dns_add(const char* ifname, int ifaceid, const char* addrPlain) {
     char arg6[256]; // address=...
     intptr_t i;
     
-    sprintf(arg5,"interface=\"%s\"", ifname);
+    sprintf(arg5,"\"%s\"", ifname);
     sprintf(arg6,"address=%s", addrPlain);
     i=_spawnl(_P_DETACH,netshPath,netshPath,arg1,arg2,arg3,arg4,arg5,arg6,NULL);
 
@@ -417,7 +495,7 @@ extern int dns_del(const char* ifname, int ifaceid, const char* addrPlain) {
     char arg5[256]; // interface=...
     char arg6[256]; // address=...
     intptr_t i;
-    sprintf(arg5,"interface=\"%s\"", ifname);
+    sprintf(arg5,"\"%s\"", ifname);
     sprintf(arg6,"address=%s", addrPlain);
     i=_spawnl(_P_DETACH,netshPath,netshPath,arg1,arg2,arg3,arg4,arg5,arg6,NULL);
 
@@ -520,7 +598,6 @@ int prefix_add(const char* ifname, int ifindex, const char* prefixPlain, int pre
     sprintf(arg8,"validlifetime=%d", valid);
 
     sprintf(buf, "%s %s %s %s %s %s %s %s %s %s", arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10);
-    printf("#### %s ####\n", buf);
     if (prefix_forwarding_enabled())
         i=_spawnl(_P_DETACH,netshPath,netshPath,arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9,arg10, NULL);
     else
@@ -568,3 +645,14 @@ int prefix_forwarding_enabled()
     // FIXME: Detect if IPv6 forwarding is enabled or not
     return 1;
 }
+
+
+void link_state_change_init(volatile struct link_state_notify_t * monitored_links, volatile int * notify)
+{
+    //FIXME: implement this
+}
+
+void link_state_change_cleanup()
+{
+   //FIXME: implement this
+}
