@@ -1,12 +1,13 @@
-/*                                                                           
- * Dibbler - a portable DHCPv6                                               
- *                                                                           
- * authors: Tomasz Mrugalski <thomson@klub.com.pl>                           
- *          Marek Senderski <msend@o2.pl>                                    
- *                                                                           
- * released under GNU GPL v2 or later licence                                
- *                                                                           
- * $Id: SrvCfgIface.cpp,v 1.43 2008-06-18 20:37:20 thomson Exp $
+/*
+ * Dibbler - a portable DHCPv6
+ *
+ * authors: Tomasz Mrugalski <thomson@klub.com.pl>
+ *          Marek Senderski <msend@o2.pl>
+ * changes: Nguyen Vinh Nghiem
+ *
+ * released under GNU GPL v2 only licence
+ *
+ * $Id: SrvCfgIface.cpp,v 1.48 2008-11-11 22:41:48 thomson Exp $
  */
 
 #include <cstdlib>
@@ -68,7 +69,7 @@ bool TSrvCfgIface::getPreferedAddrClassID(SmartPtr<TDUID> duid, SmartPtr<TIPv6Ad
     SmartPtr<TSrvCfgAddrClass> ptrClass;
     this->SrvCfgAddrClassLst.first();
     while(ptrClass=SrvCfgAddrClassLst.get()) {
-        if (ptrClass->clntPrefered(duid, clntAddr)) { 
+        if (ptrClass->clntPrefered(duid, clntAddr)) {
             classid=ptrClass->getID();
             return true;
         }
@@ -97,13 +98,13 @@ bool TSrvCfgIface::getAllowedAddrClassID(SmartPtr<TDUID> duid, SmartPtr<TIPv6Add
         }
     }
 
-    if (!cnt) 
+    if (!cnt)
 	return false; // this client is not supported by any class
 
     rnd = rand() % sum;
 
     unsigned int j=0;
-    
+
     for (unsigned int i=0; i<100;i++) {
 	j += share[i];
 	if (j>=rnd) {
@@ -185,7 +186,7 @@ void TSrvCfgIface::addClntAddr(SmartPtr<TIPv6Addr> ptrAddr) {
 	    return;
 	}
     }
-    Log(Warning) << "Unable to increase address usage: no class found for " 
+    Log(Warning) << "Unable to increase address usage: no class found for "
 		 << *ptrAddr << LogEnd;
 }
 
@@ -200,30 +201,30 @@ void TSrvCfgIface::delClntAddr(SmartPtr<TIPv6Addr> ptrAddr) {
 	    return;
 	}
     }
-    Log(Warning) << "Unable to decrease address usage: no class found for " 
+    Log(Warning) << "Unable to decrease address usage: no class found for "
 		 << *ptrAddr << LogEnd;
 }
 
-SmartPtr<TSrvCfgAddrClass> TSrvCfgIface::getRandomClass(SmartPtr<TDUID> clntDuid, 
+SmartPtr<TSrvCfgAddrClass> TSrvCfgIface::getRandomClass(SmartPtr<TDUID> clntDuid,
 							SmartPtr<TIPv6Addr> clntAddr) {
 
     unsigned long classid;
 
     // step 1: Is there a class reserved for this client?
 
-    // if there is class where client is on whitelist, it should be used rather than any other class 
+    // if there is class where client is on whitelist, it should be used rather than any other class
     // that would be also suitable
     if(this->getPreferedAddrClassID(clntDuid, clntAddr, classid)) {
       Log(Debug) << "Found prefered class for client (duid = " << *clntDuid << ", addr = "
   	        << *clntAddr << ")" << LogEnd;
       return this->getClassByID(classid);
-    } 
+    }
 
     // Get one of the normal classes
     if(this->getAllowedAddrClassID(clntDuid, clntAddr, classid)) {
-	Log(Debug) << "Prefered class for client not found, using classid=" << classid << "." << LogEnd; 
+	Log(Debug) << "Prefered class for client not found, using classid=" << classid << "." << LogEnd;
 	return this->getClassByID(classid);
-    } 
+    }
 
     // This is some kind of problem...
     Log(Error) << "No class is available for client (duid=" << clntDuid->getPlain() << ", addr="
@@ -237,7 +238,7 @@ long TSrvCfgIface::countAddrClass() {
 
 
 
-/** Prefix delegation functions 
+/** Prefix delegation functions
 
 */
 
@@ -266,7 +267,7 @@ bool TSrvCfgIface::addClntPrefix(SmartPtr<TIPv6Addr> ptrAddr) {
 	    return true;
 	}
     }
-    Log(Warning) << "Unable to increase prefix usage: no prefix found for " 
+    Log(Warning) << "Unable to increase prefix usage: no prefix found for "
 		 << *ptrAddr << LogEnd;
     return false;
 }
@@ -282,41 +283,11 @@ bool TSrvCfgIface::delClntPrefix(SmartPtr<TIPv6Addr> ptrAddr) {
 	    return true;
 	}
     }
-    Log(Warning) << "Unable to decrease address usage: no class found for " 
+    Log(Warning) << "Unable to decrease address usage: no class found for "
 		 << *ptrAddr << LogEnd;
     return false;
 }
 
-#if 0
-SmartPtr<TSrvCfgPD> TSrvCfgIface::getRandomPrefix(SmartPtr<TDUID> clntDuid, 
-							SmartPtr<TIPv6Addr> clntAddr) {
-/* this method is not used currently 
-   
-      unsigned long classid;
-
-    // step 1: Is there a class reserved for this client?
-
-    // if there is class where client is on whitelist, it should be used rather than any other class 
-    // that would be also suitable
-    if(this->getPreferedAddrClassID(clntDuid, clntAddr, classid)) {
-      Log(Debug) << "Found prefered class for client (duid = " << *clntDuid << ", addr = "
-  	        << *clntAddr << ")" << LogEnd;
-      return this->getClassByID(classid);
-    } 
-
-    // Get one of the normal classes
-    if(this->getAllowedAddrClassID(clntDuid, clntAddr, classid)) {
-	Log(Debug) << "Prefered class for client not found, using classid=" << classid << "." << LogEnd; 
-	return this->getClassByID(classid);
-    } 
-
-    // This is some kind of problem...
-    Log(Error) << "No class is available for client (duid=" << clntDuid->getPlain() << ", addr="
-	       << clntAddr->getPlain() << ")." << LogEnd;*/
-    return 0;
-}
-#endif
-
 long TSrvCfgIface::countPD() {
     return this->SrvCfgPDLst.count();
 }
@@ -354,8 +325,9 @@ void TSrvCfgIface::setOptions(SmartPtr<TSrvParsGlobalOpt> opt) {
     this->RapidCommit   = opt->getRapidCommit();
     this->Unicast       = opt->getUnicast();
     this->LeaseQuery    = opt->getLeaseQuerySupport();
-    
+
     if (opt->supportFQDN()){
+	AcceptUnknownFQDN = opt->acceptUnknownFQDN();
 #ifndef MOD_SRV_DISABLE_DNSUPDATE
 	this->setFQDNLst(opt->getFQDNLst());
 	this->setFQDNMode(opt->getFQDNMode());
@@ -377,7 +349,7 @@ void TSrvCfgIface::setOptions(SmartPtr<TSrvParsGlobalOpt> opt) {
 #endif
     	Log(Debug) <<"FQDN: revDNS zoneroot lenght set to " << this->getRevDNSZoneRootLength()<< "." << LogEnd;
     }
-  
+
     if (opt->isRelay()) {
 	this->Relay = true;
 	this->RelayName        = opt->getRelayName();
@@ -417,8 +389,9 @@ void TSrvCfgIface::setDefaults() {
     this->ID = -1;
     this->NoConfig = false;
     this->preference = 0;
-   
+
     this->FQDNSupport             = false;
+    this->AcceptUnknownFQDN       = false;
     this->PrefixDelegationSupport = false;
 }
 
@@ -463,7 +436,7 @@ int TSrvCfgIface::getRelayID() {
     return this->RelayID;
 }
 
-int TSrvCfgIface::getRelayInterfaceID() {
+SPtr<TSrvOptInterfaceID> TSrvCfgIface::getRelayInterfaceID() {
     return this->RelayInterfaceID;
 }
 
@@ -485,15 +458,15 @@ void TSrvCfgIface::setFQDNLst(List(TFQDN) *fqdn) {
     this->FQDNSupport = true;
 }
 
-/** 
+/**
  * this method tries to find a name for a client. It check client's hint, and possible reservations
  * by duid or by address
- * 
- * @param duid 
- * @param addr 
- * @param hint 
- * 
- * @return 
+ *
+ * @param duid
+ * @param addr
+ * @param hint
+ *
+ * @return
  */
 SPtr<TFQDN> TSrvCfgIface::getFQDNName(SmartPtr<TDUID> duid, SmartPtr<TIPv6Addr> addr, string hint) {
     FQDNLst.first();
@@ -501,9 +474,18 @@ SPtr<TFQDN> TSrvCfgIface::getFQDNName(SmartPtr<TDUID> duid, SmartPtr<TIPv6Addr>
     SPtr<TFQDN> bestFound=0; // best FQDN found for that client
 
     SPtr<TFQDN> foo;
+
+    bool duplicate = false ; // whether the hint exists in the FQDN list
+
     while (foo=this->FQDNLst.get()) {
+
 	if (foo->isUsed())
-	    continue;
+	{ // client sent a hint, but it is used currently
+	    if (foo->Name == hint)
+	     	   duplicate = true;
+            continue;
+	}
+
 	if (duid && (foo->Duid) && *(foo->Duid)== *duid) {
 	    Log(Debug) << "FQDN found: " << foo->Name << " using duid " << duid->getPlain() << LogEnd;
 	    return foo;
@@ -512,10 +494,11 @@ SPtr<TFQDN> TSrvCfgIface::getFQDNName(SmartPtr<TDUID> duid, SmartPtr<TIPv6Addr>
 	    Log(Debug) << "FQDN found: " << foo->Name << " using address " << addr->getPlain() << LogEnd;
 	    return foo;
 	}
-	
+
 	if (foo->Name == hint){
 	    // client asked for this name. Let's check if client is allowed to get this name.
-	    if ( (!foo->Duid) && (!foo->Addr) ) {
+	   duplicate = true;
+	   if ( (!foo->Duid) && (!foo->Addr) ) {
 		Log(Debug) << "Client's hint: " << hint << " found in fqdn list, setting fqdn to "<< foo->Name << LogEnd;
 		return foo;
 	    }
@@ -523,12 +506,29 @@ SPtr<TFQDN> TSrvCfgIface::getFQDNName(SmartPtr<TDUID> duid, SmartPtr<TIPv6Addr>
 	if (!foo->Addr && !foo->Duid) {
 	    if (!bestFound)
 		bestFound = foo;
-	    return foo;
+	 //   return foo;
 	}
     }
-    
+
+    if (!hint.empty() &&  !duplicate )
+    {
+	if (acceptUnknownFQDN()) // should the server be accepting unknown names?
+	{
+	    Log(Debug) << "Client's hint: " << hint <<" but not found in FQDN list, creating an entry  " <<LogEnd;
+	    SPtr<TFQDN> newEntry = new TFQDN(hint,false);
+	    FQDNLst.append(newEntry);
+	    Log(Debug) << "Retured FQDN  " << newEntry->Name <<LogEnd;
+	    return newEntry;
+	} else
+	{
+	    Log(Info) << "FQDN: Client sent valid hint that is not mentioned in server configuration."
+		      << " Currently server is configured to drop such hints. To accept them, please "
+		      << "add accept-unknown-fqdn in the server.conf." << LogEnd;
+	}
+    }
+
     if (bestFound) {
-	Log(Debug) << "Not reserved FQDN found: " << foo->Name << LogEnd;
+	Log(Debug) << "Not reserved FQDN found: " << bestFound->Name << LogEnd;
 	return bestFound;
     }
 
@@ -536,6 +536,16 @@ SPtr<TFQDN> TSrvCfgIface::getFQDNName(SmartPtr<TDUID> duid, SmartPtr<TIPv6Addr>
     return 0;
 }
 
+/**
+ * returns if server should accept FQDN hints that are not configured in the server.conf
+ *
+ *
+ * @return true, if unknown names should be accepted
+ */
+bool TSrvCfgIface::acceptUnknownFQDN() {
+    return AcceptUnknownFQDN;
+}
+
 SmartPtr<TDUID> TSrvCfgIface::getFQDNDuid(string name) {
     SmartPtr<TDUID> res = new TDUID();
     return res;
@@ -577,7 +587,7 @@ void TSrvCfgIface::addTAAddr() {
     this->firstTA();
     ta=this->getTA();
     if (!ta) {
-	Log(Error) << "Unable to increase TA usage. TA (temporary addresses) is not found on the " 
+	Log(Error) << "Unable to increase TA usage. TA (temporary addresses) is not found on the "
 		   << this->getFullName() << " interface." << LogEnd;
 	return;
     }
@@ -589,7 +599,7 @@ void TSrvCfgIface::delTAAddr() {
     this->firstTA();
     ta = this->getTA();
     if (!ta) {
-	Log(Error) << "Unable to decrease TA usage. TA (temporary addresses) is not found on the " 
+	Log(Error) << "Unable to decrease TA usage. TA (temporary addresses) is not found on the "
 		   << this->getFullName() << " interface." << LogEnd;
 	return;
     }
@@ -611,7 +621,7 @@ ostream& operator<<(ostream& out,TSrvCfgIface& iface) {
 
     if (iface.Relay) {
 	out << "    <relay name=\"" << iface.RelayName << "\" ifindex=\"" << iface.RelayID << "\" interfaceid=\""
-	    << iface.RelayInterfaceID << "\"/>" << std::endl;
+	    << iface.RelayInterfaceID->getPlain() << "\"/>" << std::endl;
     } else {
 	out << "    <!-- <relay/> -->" << std::endl;
     }
@@ -620,7 +630,7 @@ ostream& operator<<(ostream& out,TSrvCfgIface& iface) {
     out << "    <ifaceMaxLease>" << iface.IfaceMaxLease << "</ifaceMaxLease>" << std::endl;
     out << "    <clntMaxLease>" << iface.ClntMaxLease << "</clntMaxLease>" << std::endl;
     out << "    <LeaseQuery>" << (iface.LeaseQuery?"1":"0") << "</LeaseQuery>" << std::endl;
-       
+
     if (iface.Unicast) {
         out << "    <unicast>" << *(iface.Unicast) << "</unicast>" << endl;
     } else {
@@ -638,7 +648,7 @@ ostream& operator<<(ostream& out,TSrvCfgIface& iface) {
     SmartPtr<TSrvCfgAddrClass>	ia;
     iface.SrvCfgAddrClassLst.first();
     out << "    <!-- IA: non-temporary addr class count: " << iface.SrvCfgAddrClassLst.count() << "-->" << endl;
-    while( ia=iface.SrvCfgAddrClassLst.get() ) {	
+    while( ia=iface.SrvCfgAddrClassLst.get() ) {
 	out << *ia;
     }
 
@@ -647,7 +657,7 @@ ostream& operator<<(ostream& out,TSrvCfgIface& iface) {
     SmartPtr<TSrvCfgPD>	pd;
     iface.SrvCfgPDLst.first();
     out << "    <!-- PD: prefix delegation class count: " << iface.SrvCfgPDLst.count() << "-->" << endl;
-    while( pd=iface.SrvCfgPDLst.get() ) {	
+    while( pd=iface.SrvCfgPDLst.get() ) {
 	out << *pd;
     }
 
@@ -657,10 +667,10 @@ ostream& operator<<(ostream& out,TSrvCfgIface& iface) {
     iface.firstTA();
     out << "    <!-- TA: temporary IPv6 addr class count: " << iface.SrvCfgTALst.count() << "-->" << endl;
     while( ta=iface.getTA() )
-    {	
+    {
 	out << *ta;
     }
-    
+
     out << endl << "    <!-- options -->" << endl;
 
     // option: DNS-SERVERS
@@ -746,7 +756,7 @@ ostream& operator<<(ostream& out,TSrvCfgIface& iface) {
 	iface.VendorSpec.first();
         SPtr<TSrvOptVendorSpec> v;
 	while (v = iface.VendorSpec.get()) {
-	    out << "      <vendorSpec vendor=\"" << v->getVendor() << "\" length=\"" << v->getVendorDataLen() 
+	    out << "      <vendorSpec vendor=\"" << v->getVendor() << "\" length=\"" << v->getVendorDataLen()
 	    << "\">" << v->getVendorDataPlain() << "</vendorSpec>" << endl;
 	}
 	out << "    </vendorSpecList>" << endl;
@@ -754,6 +764,17 @@ ostream& operator<<(ostream& out,TSrvCfgIface& iface) {
 	out << "    <!-- <vendorSpec/> -->" << endl;
     }
 
+    out << "    <!-- " << iface.getExtraOptions().count() << " extra option(s) -->" << endl;
+
+    List(TSrvOptGeneric) extraOpts;
+    SPtr<TSrvOptGeneric> gen;
+    extraOpts = iface.getExtraOptions();
+    extraOpts.first();
+    while (gen = extraOpts.get())
+    {
+	out << "      <extraOption length=\"" << gen->getSize()-4 << "\"/>" << endl;
+    }
+
     // option: FQDN
     if (iface.supportFQDN()) {
       SPtr<TFQDN> f;
@@ -764,7 +785,7 @@ ostream& operator<<(ostream& out,TSrvCfgIface& iface) {
 	    out << "       " << *f;
       }
       out << "    </fqdnOptions>" << endl;
-    } else { 
+    } else {
       out << "    <!-- <fqdnOptions/> -->" << endl;
     }
 
@@ -775,7 +796,32 @@ ostream& operator<<(ostream& out,TSrvCfgIface& iface) {
     while (ex = iface.ExceptionsLst.get()) {
 	out << *ex;
     }
-    
+
     out << "  </SrvCfgIface>" << endl;
     return out;
 }
+
+void TSrvCfgIface::mapAllowDenyList( List(TSrvCfgClientClass) clientClassLst)
+{
+
+	//  Log(Info)<<"Mapping allow, deny list inside interface "<<Name<<LogEnd;
+	  SmartPtr<TSrvCfgAddrClass> ptrClass;
+	  this->SrvCfgAddrClassLst.first();
+	  while(ptrClass=SrvCfgAddrClassLst.get()){
+		  ptrClass->mapAllowDenyList(clientClassLst);
+	  }
+
+
+   // Map the Allow and Deny list to TA c
+	  SmartPtr<TSrvCfgTA> ptrTA;
+	  this->SrvCfgTALst.first();
+	  while(ptrTA = SrvCfgTALst.get()){
+		  ptrTA->mapAllowDenyList(clientClassLst);
+	  }
+	  // Map the Allow and Deny list to prefix
+	  SmartPtr<TSrvCfgPD> ptrPD;
+	  this->SrvCfgPDLst.first();
+	  while(ptrPD = SrvCfgPDLst.get()){
+		  ptrPD->mapAllowDenyList(clientClassLst);
+	  }
+}
