@@ -5,9 +5,9 @@
  *          Marek Senderski <msend@o2.pl>
  * changes: Krzysztof Wnuk
  *
- * released under GNU GPL v2 or later licence
+ * released under GNU GPL v2 only licence
  *
- * $Id: SrvCfgOptions.cpp,v 1.3 2008-03-02 19:27:21 thomson Exp $
+ * $Id: SrvCfgOptions.cpp,v 1.7 2008-08-29 00:07:33 thomson Exp $
  *
  */
 
@@ -230,6 +230,18 @@ void TSrvCfgOptions::setVendorSpec(List(TSrvOptVendorSpec) vendor) {
     this->VendorSpec = vendor;
 }
 
+void TSrvCfgOptions::setExtraOptions(List(TSrvOptGeneric) extra)
+{
+    Log(Debug) << "Setting " << extra.count() << " generic option(s)." << LogEnd;
+    ExtraOpts = extra;
+}
+
+List(TSrvOptGeneric) TSrvCfgOptions::getExtraOptions()
+{
+    return ExtraOpts;
+}
+
+
 bool TSrvCfgOptions::setOptions(SPtr<TSrvParsGlobalOpt> opt)
 {
     if (opt->supportDNSServer())  this->setDNSServerLst(opt->getDNSServerLst());
@@ -244,7 +256,12 @@ bool TSrvCfgOptions::setOptions(SPtr<TSrvParsGlobalOpt> opt)
     if (opt->supportNISPDomain()) this->setNISPDomain(opt->getNISPDomain());
     if (opt->supportLifetime())   this->setLifetime(opt->getLifetime());
     if (opt->supportVendorSpec()) this->setVendorSpec(opt->getVendorSpec());
-
+    if (opt->getTunnelMode())     
+    {
+	VendorSpecSupport = true;
+	this->VendorSpec.append(opt->getTunnelVendorSpec());
+    }
+    setExtraOptions(opt->getExtraOptions());
     return true;
 }
 
