@@ -1,11 +1,13 @@
-/*                                                                           
- * Dibbler - a portable DHCPv6                                               
- *                                                                           
- * author: Krzysztof Wnuk <keczi@poczta.onet.pl>
- * 
- * released under GNU GPL v2 or later licence
- *                                                                           
- * $Id: SrvCfgPD.cpp,v 1.9 2008-06-18 20:37:20 thomson Exp $
+/*
+ * Dibbler - a portable DHCPv6
+ *
+ * authors: Krzysztof Wnuk <keczi@poczta.onet.pl>
+ * changes: Tomasz Mrugalski
+ *          Nguyen Vinh Nghiem
+ *
+ * released under GNU GPL v2 only licence
+ *
+ * $Id: SrvCfgPD.cpp,v 1.12 2008-10-12 20:07:31 thomson Exp $
  *
  */
 
@@ -14,12 +16,12 @@
 #include "SrvParsGlobalOpt.h"
 #include "DHCPConst.h"
 #include "Logger.h"
-
+#include "SrvMsg.h"
 /*
  * static field initialization
  */
 unsigned long TSrvCfgPD::staticID=0;
- 
+
 TSrvCfgPD::TSrvCfgPD() {
     this->PD_T1Beg    = 0;
     this->PD_T1End    = DHCPV6_INFINITY;
@@ -66,10 +68,9 @@ unsigned long TSrvCfgPD::getPD_Length() {
 bool TSrvCfgPD::setOptions(SmartPtr<TSrvParsGlobalOpt> opt, int prefixLength)
 {
     int poolLength=0;
-    int cnt = 0;
-    Log(Debug) << "PD: Client will receive /" << prefixLength << " prefixes (T1=" << opt->getT1Beg() 
+    Log(Debug) << "PD: Client will receive /" << prefixLength << " prefixes (T1=" << opt->getT1Beg()
 	       << ".." << opt->getT1End() << ", T2=" << opt->getT2Beg() << ".." << opt->getT2End()
-	       << ")." <<LogEnd; 
+	       << ")." <<LogEnd;
     this->PD_T1Beg    = opt->getT1Beg();
     this->PD_T2Beg    = opt->getT2Beg();
     this->PD_T1End    = opt->getT1End();
@@ -78,21 +79,11 @@ bool TSrvCfgPD::setOptions(SmartPtr<TSrvParsGlobalOpt> opt, int prefixLength)
     this->PD_PrefEnd  = opt->getPrefEnd();
     this->PD_ValidBeg = opt->getValidBeg();
     this->PD_ValidEnd = opt->getValidEnd();
-    
+
     this->PD_Length   = prefixLength;
     PD_MaxLease = opt->getClassMaxLease();
-    
+
     SmartPtr<TStationRange> PD_Range;
-    // FIXME: Implement white-list, black-list support
-    /* For a while white and black list will be not used
-	opt->firstRejedClnt();
-    while(PD_Range=opt->getRejedClnt())
-        this->RejedClnt.append(PD_Range);
-    
-    opt->firstAcceptClnt();
-    while(PD_Range=opt->getAcceptClnt())
-        this->AcceptClnt.append(PD_Range);
-	*/
 
     opt->firstPool();
     SPtr<TStationRange> pool = 0;
@@ -103,18 +94,24 @@ bool TSrvCfgPD::setOptions(SmartPtr<TSrvParsGlobalOpt> opt, int prefixLength)
     this->PD_Count = prefixLength - pool->getPrefixLength();
     if (this->PD_Count > 31)
 	this->PD_Count = DHCPV6_INFINITY;
-    this->PD_Count = 2 << (this->PD_Count-1);
+    if (this->PD_Count!=0)
+    {
+	this->PD_Count = 2 << (this->PD_Count-1);
+    } else {
+	this->PD_Count = 1; // only 1 prefix available
+    }
 
     opt->firstPool();
     while ( pool = opt->getPool() ) {
 	poolLength = pool->getPrefixLength();
 	PoolLst.append(pool);
-	Log(Debug) << "PD: Pool " << pool->getAddrL()->getPlain() << " - " 
-		   << pool->getAddrR()->getPlain() << ", pool length: " 
+	Log(Debug) << "PD: Pool " << pool->getAddrL()->getPlain() << " - "
+		   << pool->getAddrR()->getPlain() << ", pool length: "
 		   << pool->getPrefixLength() << "." << LogEnd;
+	/* FIXME: this code is fishy. It behave erraticaly, when there is only 1 prefix to be assigned
 	if (this->PD_Count > pool->rangeCount())
 	    this->PD_Count = pool->rangeCount();
-	cnt++;
+	    cnt++; */
     }
 
     // calculate common section
@@ -125,14 +122,12 @@ bool TSrvCfgPD::setOptions(SmartPtr<TSrvParsGlobalOpt> opt, int prefixLength)
 	return false;
     }
 
-    //Log(Debug) << "#### pool->getAddrL() = " << pool->getAddrL()->getPlain() << LogEnd;
-    //Log(Debug) << "#### pool->getAddrR() = " << pool->getAddrR()->getPlain() << LogEnd;
     CommonPool = new TStationRange( new TIPv6Addr(*pool->getAddrL()), new TIPv6Addr(*pool->getAddrR()));
     CommonPool->truncate(pool->getPrefixLength()+1, prefixLength);
     CommonPool->setPrefixLength(poolLength);
 
-    /* Log(Debug) << "PD: Common part is " << CommonPool->getAddrL()->getPlain() << " - " 
-       << CommonPool->getAddrR()->getPlain() << ", pool length: " 
+    /* Log(Debug) << "PD: Common part is " << CommonPool->getAddrL()->getPlain() << " - "
+       << CommonPool->getAddrR()->getPlain() << ", pool length: "
        << CommonPool->getPrefixLength() << "." << LogEnd; */
 
     // set up prefix counter counts
@@ -140,6 +135,8 @@ bool TSrvCfgPD::setOptions(SmartPtr<TSrvParsGlobalOpt> opt, int prefixLength)
     if (this->PD_MaxLease > this->PD_Count)
 	this->PD_MaxLease = this->PD_Count;
     Log(Debug) << "PD: Up to " << this->PD_Count << " prefixes may be assigned." << LogEnd;
+    allowLst = opt->getAllowClientClassString();
+    denyLst = opt->getDenyClientClassString();
     return true;
 }
 
@@ -154,10 +151,10 @@ bool TSrvCfgPD::prefixInPool(SmartPtr<TIPv6Addr> prefix)
     return false;
 }
 
-/** 
+/**
  * returns random prefix from a first pool
- * 
- * @return 
+ *
+ * @return
  */
 SmartPtr<TIPv6Addr> TSrvCfgPD::getRandomPrefix()
 {
@@ -169,14 +166,14 @@ SmartPtr<TIPv6Addr> TSrvCfgPD::getRandomPrefix()
     return 0;
 }
 
-/** 
- * gets random prefix from the common part (b) and 
+/**
+ * gets random prefix from the common part (b) and
  * returns a list of prefixes generated by contatenation
  * of the common part and pool-specific prefix
- * 
- * @param TIPv6Addr 
- * 
- * @return 
+ *
+ * @param TIPv6Addr
+ *
+ * @return
  */
 List(TIPv6Addr) TSrvCfgPD::getRandomList() {
     SPtr<TIPv6Addr> commonPart,tmp;
@@ -233,14 +230,14 @@ unsigned long TSrvCfgPD::getTotalCount() {
 ostream& operator<<(ostream& out,TSrvCfgPD& prefix)
 {
     out << "    <PD id=\"" << prefix.ID << "\">" << std::endl;
-    out << "      <!-- total prefixes in class: " << prefix.PD_Count 
+    out << "      <!-- total prefixes in class: " << prefix.PD_Count
 	<< ", prefixes assigned: " << prefix.PD_Assigned << " -->" << endl;
     out << "      <T1 min=\"" << prefix.PD_T1Beg << "\" max=\"" << prefix.PD_T1End  << "\" />" << endl;
     out << "      <T2 min=\"" << prefix.PD_T2Beg << "\" max=\"" << prefix.PD_T2End  << "\" />" << endl;
     out << "      <prefered-lifetime min=\"" << prefix.PD_PrefBeg << "\" max=\"" << prefix.PD_PrefEnd  << "\" />" << endl;
     out << "      <valid-lifetime min=\"" << prefix.PD_ValidBeg << "\" max=\"" << prefix.PD_ValidEnd  << "\" />" << endl;
     out << "      <PDMaxLease>" << prefix.PD_MaxLease << "</PDMaxLease>" << endl;
-        
+
     SmartPtr<TStationRange> statRange;
     out << "      <!-- prefix range -->" << endl;
     SPtr<TStationRange> pool;
@@ -248,12 +245,12 @@ ostream& operator<<(ostream& out,TSrvCfgPD& prefix)
     while (pool = prefix.PoolLst.get()) {
 	out << *pool;
     }
-    
+
     /*out << "      <!-- reject-clients ranges:" << addrClass.RejedClnt.count() << " -->" << endl;
     addrClass.RejedClnt.first();
     while(statRange=addrClass.RejedClnt.get())
-	out << *statRange;	
-	
+	out << *statRange;
+
     out << "      <!-- accept-only ranges:" << addrClass.AcceptClnt.count() << " -->" << endl;
     addrClass.AcceptClnt.first();
     while(statRange=addrClass.AcceptClnt.get())
@@ -261,3 +258,93 @@ ostream& operator<<(ostream& out,TSrvCfgPD& prefix)
     out << "    </PD>" << std::endl;
     return out;
 }
+
+
+
+void TSrvCfgPD::mapAllowDenyList( List(TSrvCfgClientClass) clientClassLst)
+{
+
+	Log(Info)<<"Mapping allow, deny list to PD  "<< ID <<LogEnd;
+
+	SmartPtr<string> classname;
+	SmartPtr<TSrvCfgClientClass> clntClass;
+
+	allowLst.first();
+	while (classname = allowLst.get())
+	{
+		clientClassLst.first();
+		while( clntClass = clientClassLst.get() )
+		{
+			if (clntClass->getClassName()== *classname)
+			{
+				allowClientClassLst.append(clntClass);
+			//	Log(Info)<<"  Insert ino allow list  "<<clntClass->getClassName()<<LogEnd;
+			}
+		}
+
+	}
+
+
+	denyLst.first();
+	while (classname = denyLst.get())
+		{
+			clientClassLst.first();
+			while( clntClass = clientClassLst.get() )
+			{
+				if (clntClass->getClassName()== *classname)
+				{
+					denyClientClassLst.append(clntClass);
+				//	Log(Info)<<"  Insert ino deny list  "<<clntClass->getClassName()<<LogEnd;
+				}
+			}
+
+		}
+}
+
+
+bool TSrvCfgPD::clntSupported(SmartPtr<TDUID> duid,SmartPtr<TIPv6Addr> clntAddr, SmartPtr<TSrvMsg> msg)
+{
+
+    // is client on denied client class
+	SmartPtr<TSrvCfgClientClass> clntClass;
+	denyClientClassLst.first();
+	while(clntClass = denyClientClassLst.get())
+	{
+		if (clntClass->isStatisfy(msg))
+		return false;
+	}
+
+	// is client on accepted client class
+	allowClientClassLst.first();
+	while(clntClass = allowClientClassLst.get())
+	{
+		if (clntClass->isStatisfy(msg))
+			return true;
+	}
+
+	/*
+    SmartPtr<TStationRange> range;
+    RejedClnt.first();
+
+    // is client on black list?
+    while(range=RejedClnt.get())
+        if (range->in(duid,clntAddr))
+            return false;
+
+    if (AcceptClnt.count()) {
+        AcceptClnt.first();
+	    // there's white list
+        while(range=AcceptClnt.get()) {
+	    // is client on this white list?
+            if (range->in(duid,clntAddr))
+                return true;
+        }
+        return false;
+    }
+*/
+    if (allowClientClassLst.count())
+    	return false ;
+   return true;
+
+
+}
