@@ -4,9 +4,9 @@
  * authors: Tomasz Mrugalski <thomson@klub.com.pl>                           
  *          Marek Senderski <msend@o2.pl>                                    
  *    changes: Krzysztof Wnuk <keczi@poczta.onet.pl>                                                                        
- * released under GNU GPL v2 or later licence                                
+ * released under GNU GPL v2 only licence                                
  *                                                                           
- * $Id: SrvParsIfaceOpt.cpp,v 1.13 2007-10-25 07:00:52 thomson Exp $
+ * $Id: SrvParsIfaceOpt.cpp,v 1.18 2008-11-11 22:41:49 thomson Exp $
  *
  */
 
@@ -32,6 +32,7 @@ TSrvParsIfaceOpt::TSrvParsIfaceOpt(void)
     this->SIPServerSupport  = false;
     this->SIPDomainSupport  = false;
     this->FQDNSupport       = false;
+    this->AcceptUnknownFQDN = false;
     this->NISServerSupport  = false;
     this->NISDomainSupport  = false;
     this->NISPServerSupport = false;
@@ -39,6 +40,9 @@ TSrvParsIfaceOpt::TSrvParsIfaceOpt(void)
     this->LifetimeSupport   = false;
     this->VendorSpecSupport = false;
 
+    this->TunnelMode = 0;
+    this->TunnelVendorSpec = 0;
+
     this->Relay = false;
     this->RelayName = "[unknown]";
     this->RelayID = -1;
@@ -48,6 +52,14 @@ TSrvParsIfaceOpt::TSrvParsIfaceOpt(void)
 TSrvParsIfaceOpt::~TSrvParsIfaceOpt(void) {
 }
 
+void TSrvParsIfaceOpt::acceptUnknownFQDN(bool accept) {
+    AcceptUnknownFQDN = accept;
+}
+
+bool TSrvParsIfaceOpt::acceptUnknownFQDN() {
+    return AcceptUnknownFQDN;
+}
+
 void TSrvParsIfaceOpt::setLeaseQuerySupport(bool support)
 {
     this->LeaseQuery = support;
@@ -116,7 +128,7 @@ void TSrvParsIfaceOpt::setRelayID(int id) {
     this->RelayID    = id;
 }
 
-void TSrvParsIfaceOpt::setRelayInterfaceID(int id) {
+void TSrvParsIfaceOpt::setRelayInterfaceID(SPtr<TSrvOptInterfaceID> id) {
     this->Relay = true;
     this->RelayInterfaceID= id;
 }
@@ -129,7 +141,7 @@ int TSrvParsIfaceOpt::getRelayID() {
     return this->RelayID;
 }
 
-int TSrvParsIfaceOpt::getRelayInterfaceID() {
+SPtr<TSrvOptInterfaceID> TSrvParsIfaceOpt::getRelayInterfaceID() {
     return this->RelayInterfaceID;
 }
 
@@ -225,6 +237,18 @@ bool TSrvParsIfaceOpt::supportSIPDomain() {
     return this->SIPDomainSupport;
 }
 
+// --- option: extra ---
+List(TSrvOptGeneric) TSrvParsIfaceOpt::getExtraOptions()
+{
+    return ExtraOpts;
+}
+
+void TSrvParsIfaceOpt::setExtraOptions(List(TSrvOptGeneric) extraOpts)
+{
+    ExtraOpts = extraOpts;
+}
+
+
 // --- option: FQDN ---
 
 void TSrvParsIfaceOpt::setFQDNLst(List(TFQDN) *fqdn) {
@@ -390,3 +414,37 @@ unsigned int TSrvParsIfaceOpt::getLifetime() {
 bool TSrvParsIfaceOpt::supportLifetime() {
     return this->LifetimeSupport;
 }
+
+void TSrvParsIfaceOpt::setTunnelMode(int vendor, int mode, SPtr<TIPv6Addr> addr)
+{
+    TunnelMode = mode;
+    char tmpbuf[32];
+    int len;
+    
+    tmpbuf[0] = 0x0;
+    tmpbuf[1] = OPTION_VENDORSPEC_TUNNEL_TYPE;
+    tmpbuf[2] = 0x0;
+    tmpbuf[3] = 0x1;  // 0x0001 - option-len
+    tmpbuf[4] = mode;
+
+    tmpbuf[5] = 0x0;
+    tmpbuf[6] = OPTION_VENDORSPEC_ENDPOINT;
+    tmpbuf[7] = 0x0;
+    tmpbuf[8] = 0x10; // 0x0010 - option-code
+    memmove(tmpbuf+9,addr->getAddr(), 16);
+    len = 9+16; // actual length
+
+    SPtr<TSrvOptVendorSpec> v = new TSrvOptVendorSpec(vendor, tmpbuf, len, 0);
+
+    TunnelVendorSpec = v;
+}
+
+int TSrvParsIfaceOpt::getTunnelMode()
+{
+    return TunnelMode;
+}
+
+SPtr<TSrvOptVendorSpec> TSrvParsIfaceOpt::getTunnelVendorSpec()
+{
+    return TunnelVendorSpec;
+}
