@@ -4,9 +4,9 @@
  * authors: Tomasz Mrugalski <thomson@klub.com.pl>                           
  *          Marek Senderski <msend@o2.pl>                                    
  *  changes: Krzysztof Wnuk <keczi@poczta.onet.pl>                                                                         
- * released under GNU GPL v2 or later licence                                
+ * released under GNU GPL v2 only licence                                
  *                                                                           
- * $Id: SrvParsIfaceOpt.h,v 1.13 2007-10-25 07:00:52 thomson Exp $
+ * $Id: SrvParsIfaceOpt.h,v 1.18 2008-11-11 22:41:49 thomson Exp $
  *
  */
 
@@ -14,6 +14,8 @@
 #define TSRCPARSIFACEOPT_H_
 
 #include "SrvOptVendorSpec.h"
+#include "SrvOptGeneric.h"
+#include "SrvOptInterfaceID.h"
 #include "SrvParsClassOpt.h"
 #include "FQDN.h"
 
@@ -41,10 +43,10 @@ public:
 
     void setRelayName(string name);
     void setRelayID(int ifindex);
-    void setRelayInterfaceID(int id);
+    void setRelayInterfaceID(SPtr<TSrvOptInterfaceID> id);
     string getRelayName();
     int getRelayID();
-    int getRelayInterfaceID();
+    SPtr<TSrvOptInterfaceID> getRelayInterfaceID();
     bool isRelay();
 
     // leasequery support
@@ -89,6 +91,8 @@ public:
     string getFQDNName();
     int getRevDNSZoneRootLength();
     void setRevDNSZoneRootLength(int revDNSZoneRootLength);
+    void acceptUnknownFQDN(bool accept);
+    bool acceptUnknownFQDN();
 
     SmartPtr<TDUID> getFQDNDuid(string name);
     void setFQDNLst(List(TFQDN) *fqdn);
@@ -127,6 +131,15 @@ public:
     bool supportVendorSpec();
     List(TSrvOptVendorSpec) getVendorSpec();
 
+    // option: extra
+    void setExtraOptions(List(TSrvOptGeneric) extraOpts);
+    List(TSrvOptGeneric) getExtraOptions();
+
+    // experimental tunnel-mode
+    void setTunnelMode(int vendor, int mode, SPtr<TIPv6Addr> addr);
+    int getTunnelMode();
+    SPtr<TSrvOptVendorSpec> getTunnelVendorSpec();
+
 private:
     char Preference;
     bool RapidCommit;
@@ -139,7 +152,7 @@ private:
     bool Relay;
     string RelayName;
     int RelayID;
-    int RelayInterfaceID;
+    SPtr<TSrvOptInterfaceID> RelayInterfaceID;
 
     // options
     bool DNSServerSupport;
@@ -168,10 +181,16 @@ private:
     string NISDomain;
     string NISPDomain;
     int FQDNMode;
+    bool AcceptUnknownFQDN;
     int revDNSZoneRootLength;
     unsigned int Lifetime;
 
+    // tunnel-mode
+    int TunnelMode;
+    SPtr<TSrvOptVendorSpec> TunnelVendorSpec;
+
     List(TSrvOptVendorSpec) VendorSpec;
+    List(TSrvOptGeneric) ExtraOpts;
 };
 
 #endif
