@@ -5,9 +5,9 @@
  *          Marek Senderski <msend@o2.pl>
  * changes: Petr Pisar <petr.pisar(at)atlas(dot)cz>
  *
- * released under GNU GPL v2 or later licence
+ * released under GNU GPL v2 only licence
  *
- * $Header: /var/cvs/dibbler/SrvIfaceMgr/SrvIfaceMgr.h,v 1.9 2007-09-07 08:31:21 thomson Exp $
+ * $Header: /var/cvs/dibbler/SrvIfaceMgr/SrvIfaceMgr.h,v 1.11 2008-11-11 22:41:49 thomson Exp $
  *
  */
 
@@ -23,13 +23,6 @@ class TSrvIfaceMgr;
 #include "SrvIfaceIface.h"
 #include "SrvMsg.h"
 
-/**
- * ServerInterfaceManager - sends and receives messages (server version)
- * @date 2003-10-14
- * @author Tomasz Mrugalski <admin@klub.com.pl>
- * @licence GNU GPL v2 or later
- */
-
 class TSrvIfaceMgr :public TIfaceMgr {
  public:
     TSrvIfaceMgr(string xmlFile);
@@ -44,7 +37,7 @@ class TSrvIfaceMgr :public TIfaceMgr {
 				      SmartPtr<TIPv6Addr> peer, 
 				      char * buf, int bufsize);
     
-    bool setupRelay(string name, int ifindex, int underIfindex, int interfaceID);
+    bool setupRelay(string name, int ifindex, int underIfindex, SPtr<TSrvOptInterfaceID> interfaceID);
     void dump();
     
     // ---sends messages---
