@@ -6,9 +6,9 @@
  * changes: Krzysztof Wnuk <keczi@poczta.onet.pl>
  *          Michal Kowalczuk <michal@kowalczuk.eu>
  *
- * released under GNU GPL v2 or later licence
+ * released under GNU GPL v2 only licence
  *
- * $Id: SrvMsg.cpp,v 1.53 2008-06-18 20:38:26 thomson Exp $
+ * $Id: SrvMsg.cpp,v 1.58 2008-11-13 22:40:26 thomson Exp $
  */
 
 #include <sstream>
@@ -31,6 +31,7 @@
 #include "SrvOptServerUnicast.h"
 #include "SrvOptStatusCode.h"
 #include "SrvOptRapidCommit.h"
+#include "SrvOptGeneric.h"
 #include "SrvOptLQ.h"
 #include "SrvOptTA.h"
 #include "SrvCfgOptions.h"
@@ -129,13 +130,17 @@ TSrvMsg::TSrvMsg(SmartPtr<TSrvIfaceMgr> IfaceMgr,
 		       << " bytes left to parse. Bytes ignored." << LogEnd;
 	    break;
 	}
-        unsigned short code   = ntohs( *((unsigned short*) (buf+pos)));
+
+
+        unsigned short code   = buf[pos]*256 + buf[pos+1];
         pos+=2;
-        unsigned short length = ntohs( *((unsigned short*) (buf+pos)));
+        unsigned short length = buf[pos]*256 + buf[pos+1];
         pos+=2;
+
 	if (pos+length>bufSize) {
-	    Log(Error) << "Invalid option (type=" << code << ", len=" << length 
-		       << " received (msgtype=" << MsgType << "). Message dropped." << LogEnd;
+	    Log(Error) << "Malformed option (type=" << code << ", len=" << length 
+		       << ", offset from beginning of DHCPv6 message=" << pos+4 /* 4=msg-type+trans-id */
+		       << ") received (msgtype=" << MsgType << "). Message dropped." << LogEnd;
 	    IsDone = true;
 	    return;
 	}
@@ -191,7 +196,7 @@ TSrvMsg::TSrvMsg(SmartPtr<TSrvIfaceMgr> IfaceMgr,
 	case OPTION_DOMAIN_LIST:
 	    ptr = new TSrvOptDomainName(buf+pos, length,this);
 	    break;
-	case OPTION_TIME_ZONE:
+	case OPTION_NEW_TZDB_TIMEZONE:
 	    ptr = new TSrvOptTimeZone(buf+pos, length,this);
 	    break;
 	case OPTION_SIP_SERVER_A:
@@ -557,7 +562,7 @@ bool TSrvMsg::appendRequestedOptions(SmartPtr<TDUID> duid, SmartPtr<TIPv6Addr> a
     };
     
     // --- option: TIMEZONE --- 
-    if ( reqOpts->isOption(OPTION_TIME_ZONE) && ptrIface->supportTimezone() ) {
+    if ( reqOpts->isOption(OPTION_NEW_TZDB_TIMEZONE) && ptrIface->supportTimezone() ) {
 	SmartPtr<TSrvOptTimeZone> optTimezone;
 	if (ex && ex->supportTimezone())
 	    optTimezone = new TSrvOptTimeZone(ex->getTimezone(),this);
@@ -654,6 +659,18 @@ bool TSrvMsg::appendRequestedOptions(SmartPtr<TDUID> duid, SmartPtr<TIPv6Addr> a
 	newOptionAssigned = true;
     }
 
+    // --- option: extra options ---
+    List(TSrvOptGeneric) generics = ptrIface->getExtraOptions();
+
+    SPtr<TSrvOptGeneric> gen;
+    generics.first();
+    while (gen = generics.get())
+    {
+	Log(Debug) << "Experimental: Appending extra option " << gen->getSize() << LogEnd;
+	Options.append( (Ptr*) gen);
+	newOptionAssigned = true;
+    }
+
     // --- option: KEYGEN ---
 #ifndef MOD_DISABLE_AUTH
     if ( reqOpts->isOption(OPTION_KEYGEN) && SrvCfgMgr->getDigest() != DIGEST_NONE ) { // && this->MsgType == ADVERTISE_MSG ) {
