@@ -1,12 +1,13 @@
-/*                                                                           
- * Dibbler - a portable DHCPv6                                               
- *                                                                           
- * authors: Tomasz Mrugalski <thomson@klub.com.pl>                           
- *          Marek Senderski <msend@o2.pl>                                    
- *                                                                           
- * released under GNU GPL v2 or later licence                                
- *                                                                           
- * $Id: SrvOptIA_NA.cpp,v 1.27 2008-06-01 18:29:04 thomson Exp $
+/*
+ * Dibbler - a portable DHCPv6
+ *
+ * authors: Tomasz Mrugalski <thomson@klub.com.pl>
+ *          Marek Senderski <msend@o2.pl>
+ * changes: Nguyen Vinh Nghiem
+ *
+ * released under GNU GPL v2 only licence
+ *
+ * $Id: SrvOptIA_NA.cpp,v 1.30 2008-10-12 20:16:14 thomson Exp $
  */
 
 #ifdef WIN32
@@ -14,7 +15,7 @@
 #endif
 #ifdef LINUX
 #include <netinet/in.h>
-#endif 
+#endif
 
 #include "SrvOptIA_NA.h"
 #include "SrvOptIAAddress.h"
@@ -28,7 +29,7 @@
 
 TSrvOptIA_NA::TSrvOptIA_NA( long IAID, long T1, long T2, TMsg* parent)
     :TOptIA_NA(IAID,T1,T2, parent) {
-    
+
 }
 
 TSrvOptIA_NA::TSrvOptIA_NA( long IAID, long T1, long T2, int Code, string Text, TMsg* parent)
@@ -42,30 +43,30 @@ TSrvOptIA_NA::TSrvOptIA_NA( long IAID, long T1, long T2, int Code, string Text,
 TSrvOptIA_NA::TSrvOptIA_NA( char * buf, int bufsize, TMsg* parent)
     :TOptIA_NA(buf,bufsize, parent) {
     int pos=0;
-    while(pos<bufsize) 
+    while (pos<bufsize)
     {
         int code=buf[pos]*256+buf[pos+1];
         pos+=2;
         int length=buf[pos]*256+buf[pos+1];
         pos+=2;
         if ((code>0)&&(code<=24))
-        {                
+        {
             if(allowOptInOpt(parent->getType(),OPTION_IA_NA,code)) {
                 SmartPtr<TOpt> opt;
 		opt = SmartPtr<TOpt>(); /* NULL */
                 switch (code)
                 {
                 case OPTION_IAADDR:
-                    opt = (Ptr*)SmartPtr<TSrvOptIAAddress> 
+                    opt = (Ptr*)SmartPtr<TSrvOptIAAddress>
 			(new TSrvOptIAAddress(buf+pos,length,this->Parent));
                     break;
                 case OPTION_STATUS_CODE:
-                    opt = (Ptr*)SmartPtr<TSrvOptStatusCode> 
+                    opt = (Ptr*)SmartPtr<TSrvOptStatusCode>
 			(new TSrvOptStatusCode(buf+pos,length,this->Parent));
                     break;
                 default:
 		    Log(Warning) <<"Option " << code<< "not supported "
-				 <<" in  message (type=" << parent->getType() 
+				 <<" in  message (type=" << parent->getType()
 				 <<") in this version of server." << LogEnd;
                     break;
                 }
@@ -73,7 +74,7 @@ TSrvOptIA_NA::TSrvOptIA_NA( char * buf, int bufsize, TMsg* parent)
                     SubOptions.append(opt);
             }
             else {
-		Log(Warning) << "Illegal option received (type=" << code 
+		Log(Warning) << "Illegal option received (type=" << code
 			     << ") in an IA_NA option." << LogEnd;
 	    }
         }
@@ -85,24 +86,24 @@ TSrvOptIA_NA::TSrvOptIA_NA( char * buf, int bufsize, TMsg* parent)
     }
 }
 
-/** 
+/**
  * This constructor is used to create IA option as an aswer to a SOLICIT, SOLICIT (RAPID_COMMIT) or REQUEST
- * 
- * @param addrMgr 
- * @param cfgMgr 
- * @param queryOpt 
- * @param clntDuid 
- * @param clntAddr 
- * @param iface 
- * @param msgType 
- * @param parent 
+ *
+ * @param addrMgr
+ * @param cfgMgr
+ * @param queryOpt
+ * @param clntDuid
+ * @param clntAddr
+ * @param iface
+ * @param msgType
+ * @param parent
  */
 TSrvOptIA_NA::TSrvOptIA_NA(SmartPtr<TSrvAddrMgr> addrMgr,  SmartPtr<TSrvCfgMgr> cfgMgr,
 			   SmartPtr<TSrvOptIA_NA> queryOpt,
-			   SmartPtr<TDUID> clntDuid, SmartPtr<TIPv6Addr> clntAddr, 
+			   SmartPtr<TDUID> clntDuid, SmartPtr<TIPv6Addr> clntAddr,
 			   int iface, int msgType, TMsg* parent)
     :TOptIA_NA(queryOpt->getIAID(), DHCPV6_INFINITY, DHCPV6_INFINITY, parent) {
-    
+
     this->AddrMgr   = addrMgr;
     this->CfgMgr    = cfgMgr;
     this->ClntDuid  = clntDuid;
@@ -114,7 +115,7 @@ TSrvOptIA_NA::TSrvOptIA_NA(SmartPtr<TSrvAddrMgr> addrMgr,  SmartPtr<TSrvCfgMgr>
 
     // --- check if client already has binding
     if (renew(queryOpt, false)) {
-      Log(Info) << "Previous binding for client " << ClntDuid->getPlain() << ", IA(iaid=" 
+      Log(Info) << "Previous binding for client " << ClntDuid->getPlain() << ", IA(iaid="
                 << queryOpt->getIAID() << ") found and renewed." << LogEnd;
       return;
     }
@@ -146,7 +147,7 @@ TSrvOptIA_NA::TSrvOptIA_NA(SmartPtr<TSrvAddrMgr> addrMgr,  SmartPtr<TSrvCfgMgr>
 					    this->Parent);
       }
       this->SubOptions.append((Ptr*)ptrStatus);
-      
+
       return;
     }
 
@@ -165,7 +166,7 @@ TSrvOptIA_NA::TSrvOptIA_NA(SmartPtr<TSrvAddrMgr> addrMgr,  SmartPtr<TSrvCfgMgr>
     willAssign = addrsRequested;
 
     if (willAssign > addrsMax - addrsAssigned) {
-      Log(Notice) << "Client got " << addrsAssigned << " and requested " 
+      Log(Notice) << "Client got " << addrsAssigned << " and requested "
                   << addrsRequested << " more, but limit for a client is "
                   << addrsMax << LogEnd;
       willAssign = addrsMax - addrsAssigned;
@@ -177,7 +178,7 @@ TSrvOptIA_NA::TSrvOptIA_NA(SmartPtr<TSrvAddrMgr> addrMgr,  SmartPtr<TSrvCfgMgr>
       willAssign = addrsAvail;
     }
 
-    Log(Info) << "Client has " << addrsAssigned << " addrs, asks for " 
+    Log(Info) << "Client has " << addrsAssigned << " addrs, asks for "
               << addrsRequested << ", " << addrsAvail << " is available, limit for client is "
               << addrsMax << ", " << willAssign << " will be assigned." << LogEnd;
 
@@ -187,11 +188,11 @@ TSrvOptIA_NA::TSrvOptIA_NA(SmartPtr<TSrvAddrMgr> addrMgr,  SmartPtr<TSrvCfgMgr>
     SmartPtr<TOptIAAddress> optAddr;
     SmartPtr<TSrvCfgAddrClass> ptrClass;
     bool ok=true;
-    
+
     queryOpt->firstOption();
     while ( opt = queryOpt->getOption() ) {
 	switch ( opt->getOptType() ) {
-	case OPTION_IAADDR: 
+	case OPTION_IAADDR:
 	{
 	    optAddr = (Ptr*) opt;
 	    hint    = optAddr->getAddr();
@@ -200,10 +201,10 @@ TSrvOptIA_NA::TSrvOptIA_NA(SmartPtr<TSrvAddrMgr> addrMgr,  SmartPtr<TSrvCfgMgr>
 		SPtr<TIPv6Addr> cliHint = hint;
 		hint = getExceptionAddr();
 		Log(Info) << "Client requested " << cliHint->getPlain();
-		Log(Cont) << ", but there is address reserved for this client: " << hint->getPlain() 
+		Log(Cont) << ", but there is address reserved for this client: " << hint->getPlain()
 			  << " (client's hint ignored)." << LogEnd;
 	    }
-	    
+
 	    if (willAssign) {
 		// we've got free addrs left, assign one of them
 		// always register this address as used by this client
@@ -213,29 +214,29 @@ TSrvOptIA_NA::TSrvOptIA_NA(SmartPtr<TSrvAddrMgr> addrMgr,  SmartPtr<TSrvCfgMgr>
 		this->assignAddr(hint, pref, valid, quiet);
 		willAssign--;
 		addrsAssigned++;
-		
+
 	    } else {
 		ok = false;
 	    }
 	    break;
 	}
-	case OPTION_STATUS_CODE: 
+	case OPTION_STATUS_CODE:
 	{
 	    SmartPtr<TOptStatusCode> ptrStatus = (Ptr*) opt;
-	    Log(Notice) << "Receviced STATUS_CODE code=" 
+	    Log(Notice) << "Receviced STATUS_CODE code="
 			<<  ptrStatus->getCode() << ", message=(" << ptrStatus->getText()
 			<< ")" << LogEnd;
 	    break;
 	}
-	default: 
+	default:
 	{
-	    Log(Warning) << "Invalid suboption (" << opt->getOptType() 
+	    Log(Warning) << "Invalid suboption (" << opt->getOptType()
 			 << ") in an OPTION_IA_NA option received. Option ignored." << LogEnd;
 	    break;
 	}
 	}
     }
-    
+
     // --- now include STATUS CODE ---
     SmartPtr<TSrvOptStatusCode> ptrStatus;
     if (ok) {
@@ -253,7 +254,7 @@ TSrvOptIA_NA::TSrvOptIA_NA(SmartPtr<TSrvAddrMgr> addrMgr,  SmartPtr<TSrvCfgMgr>
 
     }
     SubOptions.append((Ptr*)ptrStatus);
-    
+
     // if this is a ADVERTISE message, release those addresses in TSrvMsgAdvertise::answer() method
 }
 
@@ -280,7 +281,7 @@ SmartPtr<TSrvOptIAAddress> TSrvOptIA_NA::assignAddr(SmartPtr<TIPv6Addr> hint, un
     SmartPtr<TIPv6Addr> addr;
     SmartPtr<TSrvOptIAAddress> optAddr;
     SmartPtr<TSrvCfgAddrClass> ptrClass;
-    
+
     // get address
     addr = this->getFreeAddr(hint);
     if (!addr) {
@@ -297,25 +298,25 @@ SmartPtr<TSrvOptIAAddress> TSrvOptIA_NA::assignAddr(SmartPtr<TIPv6Addr> hint, un
     }
     SubOptions.append((Ptr*)optAddr);
 
-    Log(Info) << "Client requested " << *hint <<", got " << *addr 
+    Log(Info) << "Client requested " << *hint <<", got " << *addr
 	      << " (IAID=" << this->IAID << ", pref=" << pref << ",valid=" << valid << ")." << LogEnd;
 
     // configure this IA
     this->T1= ptrClass->getT1(this->T1);
     this->T2= ptrClass->getT2(this->T2);
-    
+
     // register this address as used by this client
-    this->AddrMgr->addClntAddr(this->ClntDuid, this->ClntAddr, this->Iface, this->IAID, 
+    this->AddrMgr->addClntAddr(this->ClntDuid, this->ClntAddr, this->Iface, this->IAID,
 			       this->T1, this->T2, addr, pref, valid, quiet);
     this->CfgMgr->addClntAddr(this->Iface, addr);
-    
+
     return optAddr;
 }
 
-/** 
+/**
  * tries to find address reserved for this particular client
- * 
- * @return 
+ *
+ * @return
  */
 SPtr<TIPv6Addr> TSrvOptIA_NA::getExceptionAddr()
 {
@@ -364,7 +365,6 @@ TSrvOptIA_NA::TSrvOptIA_NA( SmartPtr<TSrvCfgMgr> cfgMgr,
     case REQUEST_MSG:
         //this->request(cfgMgr, addrMgr, queryOpt, clntAddr, clntDUID, iface, addrCount);
         break;
-    
     case RENEW_MSG:
         this->renew(queryOpt, true);
         break;
@@ -381,7 +381,7 @@ TSrvOptIA_NA::TSrvOptIA_NA( SmartPtr<TSrvCfgMgr> cfgMgr,
         this->decline(queryOpt, addrCount);
         break;
     default: {
-	Log(Warning) << "Unknown message type (" << msgType 
+	Log(Warning) << "Unknown message type (" << msgType
 		     << "). Cannot generate OPTION_IA_NA."<< LogEnd;
         SubOptions.append(new TSrvOptStatusCode(STATUSCODE_UNSPECFAIL,
 						"Unknown message type.",this->Parent));
@@ -390,12 +390,11 @@ TSrvOptIA_NA::TSrvOptIA_NA( SmartPtr<TSrvCfgMgr> cfgMgr,
     }
 }
 
-// 
-/** 
+/**
  * generate OPTION_IA_NA based on OPTION_IA_NA received in RENEW message
- * 
+ *
  * @param queryOpt - IA_NA option in the RENEW message
- * @param addrCount 
+ * @param addrCount
  *
  * @return true - if binding was renewed, false - if not found or invalid
  */
@@ -431,7 +430,7 @@ bool TSrvOptIA_NA::renew(SmartPtr<TSrvOptIA_NA> queryOpt, bool complainIfMissing
     ptrIA->setTimestamp();
     this->T1 = ptrIA->getT1();
     this->T2 = ptrIA->getT2();
-    
+
     // send addr info to client
     SmartPtr<TAddrAddr> ptrAddr;
     ptrIA->firstAddr();
@@ -505,6 +504,44 @@ void TSrvOptIA_NA::release(SmartPtr<TSrvOptIA_NA> queryOpt,
 void TSrvOptIA_NA::confirm(SmartPtr<TSrvOptIA_NA> queryOpt,
                            unsigned long &addrCount)
 {
+    SmartPtr<TSrvOptIA_NA> ia = queryOpt;
+    SmartPtr<TOpt> subOpt;
+    bool NotOnLink = false;
+
+    ia->firstOption();
+    while ( subOpt = ia->getOption() ) {
+	if (subOpt->getOptType() != OPTION_IAADDR)
+	    continue;
+
+        SmartPtr<TSrvOptIAAddress> optAddr = (Ptr*)subOpt;
+
+	//FIXME: proper check if the addresses are valid or not should be performed
+        SmartPtr<TSrvCfgAddrClass> ptrClass;
+        ptrClass = CfgMgr->getClassByAddr(this->Iface, optAddr->getAddr());
+	if (!ptrClass)
+	{
+	    NotOnLink = true;
+	    break;
+	}
+
+	// set IA Address suboptions and IA
+        optAddr->setPref( ptrClass->getPref(DHCPV6_INFINITY) );
+        optAddr->setValid( ptrClass->getValid(DHCPV6_INFINITY) );
+
+        this->setT1( ptrClass->getT1(DHCPV6_INFINITY) );
+        this->setT2( ptrClass->getT2(DHCPV6_INFINITY) );
+
+        SmartPtr<TOptIAAddress> myOptAddr;
+        myOptAddr = new TSrvOptIAAddress(optAddr->getAddr(), optAddr->getPref(),
+                                       optAddr->getValid(),this->Parent);
+        SubOptions.append( (Ptr*)myOptAddr );
+    }
+
+
+    if (NotOnLink)
+        SubOptions.append(new TSrvOptStatusCode(STATUSCODE_NOTONLINK,
+						"Those addresses are not valid on this link.",this->Parent ));
+
 }
 
 void TSrvOptIA_NA::decline(SmartPtr<TSrvOptIA_NA> queryOpt,
@@ -521,30 +558,35 @@ bool TSrvOptIA_NA::doDuties()
  * gets free address for a client
  */
 SmartPtr<TIPv6Addr> TSrvOptIA_NA::getFreeAddr(SmartPtr<TIPv6Addr> hint) {
+
+    // Getting out the request Message
     
+    SmartPtr<TSrvTransMgr> srvTrans =  ((TSrvMsg*)Parent)->SrvTransMgr;
+    SmartPtr<TSrvMsg> requestMsg =  (Ptr*)(srvTrans->requestMsg);
+
     bool invalidAddr = false;
     SmartPtr<TSrvCfgIface> ptrIface;
     SmartPtr<TIPv6Addr>    addr;
     ptrIface = this->CfgMgr->getIfaceByID(this->Iface);
     if (!ptrIface) {
-	Log(Error) << "Trying to find free address on non-existent interface (id=%d)\n" 
+	Log(Error) << "Trying to find free address on non-existent interface (id=%d)\n"
 		   << this->Iface << LogEnd;
 	return 0; // NULL
     }
-    
+
     // check if this address is ok
-    
+
     // is it anyaddress (::)?
     SmartPtr<TIPv6Addr> anyaddr = new TIPv6Addr();
     if (*anyaddr==*hint) {
-	Log(Debug) << "Client requested unspecified (" << *hint 
+	Log(Debug) << "Client requested unspecified (" << *hint
 		   << ") address. Hint ignored." << LogEnd;
 	invalidAddr = true;
     }
-    
+
     // is it multicast address (ff...)?
     if ((*(hint->getAddr()))==0xff) {
-	Log(Debug) << "Client requested multicast (" << *hint 
+	Log(Debug) << "Client requested multicast (" << *hint
 		   << ") address. Hint ignored." << LogEnd;
 	invalidAddr = true;
     }
@@ -562,29 +604,36 @@ SmartPtr<TIPv6Addr> TSrvOptIA_NA::getFreeAddr(SmartPtr<TIPv6Addr> hint) {
 	SmartPtr<TSrvCfgAddrClass> ptrClass;
 	ptrClass = this->CfgMgr->getClassByAddr(this->Iface, hint);
 
-	// best case: address belongs to supported class, and is free
-	if ( ptrClass && AddrMgr->addrIsFree(hint) ) {
-	    Log(Debug) << "Requested address (" << *hint << ") is free, great!" << LogEnd;
-	    return hint;
-	} 
+	// For Supportting Client Classify
+	// Should get clientMessage, instead of Parent message
 
-	// medium case: addess belongs to supported class, but is used
-	if ( ptrClass ) {
-	    if (ptrClass->getAssignedCount()>=ptrClass->getClassMaxLease()) {
-		Log(Debug) << "Requested address (" << *hint 
-			   << ") belongs to supported class, which has reached its limit (" 
-			   << ptrClass->getAssignedCount() << " assigned, " 
+	// If the Class are valid and support the Client (based on duid, addr, clientclass)
+	if ( ptrClass && ptrClass->clntSupported(ClntDuid, ClntAddr, requestMsg ))
+	{
+		// best case: address belongs to supported class, and is free
+		if ( AddrMgr->addrIsFree(hint) ) {
+		    Log(Debug) << "Requested address (" << *hint << ") is free, great!" << LogEnd;
+		    return hint;
+		}
+
+		// medium case: addess belongs to supported class, but is used
+		// however the class pool is still free
+
+		if (ptrClass->getAssignedCount()>=ptrClass->getClassMaxLease()) {
+		Log(Debug) << "Requested address (" << *hint
+			   << ") belongs to supported class, which has reached its limit ("
+			   << ptrClass->getAssignedCount() << " assigned, "
 			   << ptrClass->getClassMaxLease() << " max lease)." << LogEnd;
-	    } else {
-		Log(Debug) << "Requested address (" << *hint 
-			   << ") belongs to supported class, but is used." << LogEnd;
-		do {
-		    addr = ptrClass->getRandomAddr();
-		} while (!AddrMgr->addrIsFree(addr));
-		return addr;
-	    }
-	}
-    }
+		} else {
+		    Log(Debug) << "Requested address (" << *hint
+			       << ") belongs to supported class, but is used." << LogEnd;
+		    do {
+			addr = ptrClass->getRandomAddr();
+		    } while (!AddrMgr->addrIsFree(addr));
+		    return addr;
+		}
+	}// If the Class are valid and support the Client (based on duid, addr, clientclass)
+    } //  if ( !invalidAddr )
 
     // do we have a cached address for that client?
     if (addr = AddrMgr->getCachedAddr(this->ClntDuid)) {
@@ -597,28 +646,33 @@ SmartPtr<TIPv6Addr> TSrvOptIA_NA::getFreeAddr(SmartPtr<TIPv6Addr> hint) {
 	} else {
 	    Log(Warning) << "Cache: Cached address " << *addr << " found, but it is no longer valid." << LogEnd;
 	    AddrMgr->delCachedAddr(addr);
-	}
-    }
-    
+	}// else
+    }// if have cached address for that client
+
     // worst case: address does not belong to supported class
     // or specified hint is invalid
     SmartPtr<TSrvCfgAddrClass> ptrClass;
+
+
     ptrClass = ptrIface->getRandomClass(this->ClntDuid, this->ClntAddr);
-    if (!ptrClass || (ptrClass->getAssignedCount()>=ptrClass->getClassMaxLease()) ) {
+
+    if (!ptrClass || 
+	!ptrClass->clntSupported(ClntDuid, ClntAddr, requestMsg ) ||  
+	(ptrClass->getAssignedCount()>=ptrClass->getClassMaxLease()) ) {
 	// random class in invalid, let's try to find another one
-	
+
 	ptrIface->firstAddrClass();
 	while (ptrClass = ptrIface->getAddrClass()) {
-	    if (!ptrClass->clntSupported(ClntDuid, ClntAddr))
+	    if (!ptrClass->clntSupported(ClntDuid, ClntAddr, requestMsg))
 		continue;
 	    if (ptrClass->getAssignedCount()>=ptrClass->getClassMaxLease())
 		continue;
 	    break;
 	}
     }
-	
+
     if (!ptrClass) {
-	Log(Warning) << "Unable to find any non-full class for this client." << LogEnd;
+	Log(Warning) << "Unable to find any suitable (allowed, non-full) class for this client." << LogEnd;
 	return 0;
     }
 
